@@ -17,27 +17,43 @@
 # is relative to the documentation root, use os.path.abspath to make it
 # absolute, like shown here.
 #sys.path.append(os.path.abspath('some/directory'))
+
+from __future__ import absolute_import, print_function, division
+
 import os
 import sys
+import theano
+
 theano_path = os.path.join(os.path.dirname(__file__), os.pardir)
 sys.path.append(os.path.abspath(theano_path))
+import versioneer
 
 # General configuration
 # ---------------------
 
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.todo', 'sphinx.ext.doctest', 'sphinx.ext.napoleon']
+extensions = ['sphinx.ext.autodoc',
+              'sphinx.ext.todo',
+              'sphinx.ext.doctest',
+              'sphinx.ext.napoleon',
+              'sphinx.ext.linkcode']
 
 todo_include_todos = True
 napoleon_google_docstring = False
 napoleon_include_special_with_doc = False
 
+# We do it like this to support multiple sphinx version without having warning.
+# Our buildbot consider warning as error.
 try:
-    from sphinx.ext import pngmath
-    extensions.append('sphinx.ext.pngmath')
+    from sphinx.ext import imgmath
+    extensions.append('sphinx.ext.imgmath')
 except ImportError:
-    pass
+    try:
+        from sphinx.ext import pngmath
+        extensions.append('sphinx.ext.pngmath')
+    except ImportError:
+        pass
 
 
 # Add any paths that contain templates here, relative to this directory.
@@ -51,15 +67,22 @@ master_doc = 'index'
 
 # General substitutions.
 project = 'Theano'
-copyright = '2008--2016, LISA lab'
+copyright = '2008--2020, LISA lab'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
-# The short X.Y version.
-version = '0.8'
+
+# We need this hokey-pokey because versioneer needs the current
+# directory to be the root of the project to work.
+_curpath = os.getcwd()
+os.chdir(os.path.dirname(os.path.dirname(__file__)))
 # The full version, including alpha/beta/rc tags.
-release = '0.8.2'
+release = versioneer.get_version()
+# The short X.Y version.
+version = '.'.join(release.split('.')[:2])
+os.chdir(_curpath)
+del _curpath
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -100,7 +123,20 @@ pygments_style = 'sphinx'
 # must exist either in Sphinx' static/ path, or in one of the custom paths
 # given in html_static_path.
 #html_style = 'default.css'
-html_theme = 'sphinxdoc'
+# html_theme = 'sphinxdoc'
+
+# Read the docs style:
+if os.environ.get('READTHEDOCS') != 'True':
+    try:
+        import sphinx_rtd_theme
+    except ImportError:
+        pass  # assume we have sphinx >= 1.3
+    else:
+        html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
+    html_theme = 'sphinx_rtd_theme'
+
+def setup(app):
+    app.add_stylesheet("fix_rtd.css")
 
 # The name for this set of Sphinx documents.  If None, it defaults to
 # "<project> v<release> documentation".
@@ -111,8 +147,7 @@ html_theme = 'sphinxdoc'
 
 # The name of an image file (within the static path) to place at the top of
 # the sidebar.
-#html_logo = 'images/theano_logo-200x67.png'
-html_logo = 'images/theano_logo_allblue_200x46.png'
+html_logo = 'images/theano_logo_allwhite_210x70.png'
 
 # The name of an image file (within the static path) to use as favicon of the
 # docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
@@ -162,15 +197,49 @@ html_use_smartypants = True
 # Output file base name for HTML help builder.
 htmlhelp_basename = 'theanodoc'
 
+# Options for the linkcode extension
+# ----------------------------------
+# Resolve function
+# This function is used to populate the (source) links in the API
+def linkcode_resolve(domain, info):
+    def find_source():
+        # try to find the file and line number, based on code from numpy:
+        # https://github.com/numpy/numpy/blob/master/doc/source/conf.py#L286
+        obj = sys.modules[info['module']]
+        for part in info['fullname'].split('.'):
+            obj = getattr(obj, part)
+        import inspect
+        import os
+        fn = inspect.getsourcefile(obj)
+        fn = os.path.relpath(fn, start=os.path.dirname(theano.__file__))
+        source, lineno = inspect.getsourcelines(obj)
+        return fn, lineno, lineno + len(source) - 1
+
+    if domain != 'py' or not info['module']:
+        return None
+    try:
+        filename = 'theano/%s#L%d-L%d' % find_source()
+    except Exception:
+        filename = info['module'].replace('.', '/') + '.py'
+    import subprocess
+    tag = subprocess.Popen(['git', 'rev-parse', 'HEAD'],
+                           stdout=subprocess.PIPE,
+                           universal_newlines=True).communicate()[0][:-1]
+    return "https://github.com/Theano/theano/blob/%s/%s" % (tag, filename)
 
 # Options for LaTeX output
 # ------------------------
 
-# The paper size ('letter' or 'a4').
-#latex_paper_size = 'letter'
+latex_elements = {
+    # The paper size ('letter' or 'a4').
+    #latex_paper_size = 'letter',
 
-# The font size ('10pt', '11pt' or '12pt').
-latex_font_size = '11pt'
+    # The font size ('10pt', '11pt' or '12pt').
+    'pointsize': '11pt',
+
+    # Additional stuff for the LaTeX preamble.
+    #latex_preamble = '',
+}
 
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class
@@ -189,9 +258,6 @@ latex_logo = 'images/theano_logo_allblue_200x46.png'
 # not chapters.
 #latex_use_parts = False
 
-# Additional stuff for the LaTeX preamble.
-#latex_preamble = ''
-
 # Documents to append as an appendix to all manuals.
 #latex_appendices = []
 
