@@ -1,4 +1,4 @@
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 import numpy, theano
 import theano.misc.pycuda_init
 from pycuda.compiler import SourceModule
