@@ -1,4 +1,4 @@
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 import theano
 a = theano.tensor.vector("a") # declare variable
 b = a + a**10                 # build symbolic expression
