@@ -2,7 +2,7 @@
 Driver of graph construction, optimization, and linking.
 
 """
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 
 import copy
 from six import string_types, iteritems, iterkeys
@@ -12,16 +12,15 @@ import six.moves.cPickle as pickle
 from itertools import chain
 import time
 import warnings
-import numpy
+import numpy as np
 
 import theano
 from theano import config, gof
-from functools import partial
 from theano.compat import izip
 from theano.gof import graph
-import theano.compile.mode
+import theano.compile.profiling
 from theano.compile.io import (
-    In, SymbolicInput, SymbolicInputKit, SymbolicOutput)
+    In, SymbolicInput, SymbolicOutput)
 from theano.compile.ops import deep_copy_op, view_op
 from theano.gof.graph import is_same_graph
 from theano.gof.op import ops_with_inner_function
@@ -88,7 +87,7 @@ def infer_reuse_pattern(fgraph, outputs_to_disown):
     as any of the specified variables. Used internally by function,
     FunctionMaker.
 
-    This list (or set) is also refered to as no_recycling sometimes,
+    This list (or set) is also referred to as no_recycling sometimes,
     especially by linker code.
 
     """
@@ -133,6 +132,11 @@ class Supervisor:
         self.protected = list(protected)
 
     def validate(self, fgraph):
+        if config.cycle_detection == 'fast' and hasattr(fgraph, 'has_destroyers'):
+            if fgraph.has_destroyers(self.protected):
+                raise gof.InconsistencyError("Trying to destroy a protected"
+                                             "Variable.")
+            return True
         if not hasattr(fgraph, 'destroyers'):
             return True
         for r in self.protected + list(fgraph.outputs):
@@ -191,7 +195,7 @@ def std_fgraph(input_specs, output_specs, accept_inplace=False):
                    for spec, input in zip(input_specs, fgraph.inputs)
                    if not (spec.mutable or
                            (hasattr(fgraph, 'destroyers') and
-                            fgraph.destroyers(input)))))
+                            fgraph.has_destroyers([input])))))
 
     # If named nodes are replaced, keep the name
     for feature in std_fgraph.features:
@@ -286,7 +290,7 @@ class Function(object):
 
     indices = None
     """
-    List of (SymbolicInput|SymbolicInputKit, indices, [SymbolicInput,...]),
+    List of (SymbolicInput, indices, [SymbolicInput,...]),
     one tuple for each input.
 
     The first tuple element is the SymbolicInput object for the corresponding
@@ -360,7 +364,8 @@ class Function(object):
     """
 
     def __init__(self, fn, input_storage, output_storage, indices, outputs,
-                 defaults, unpack_single, return_none, output_keys, maker):
+                 defaults, unpack_single, return_none, output_keys, maker,
+                 name=None):
         self.fn = fn
         self.input_storage = input_storage
         self.output_storage = output_storage
@@ -372,10 +377,23 @@ class Function(object):
         self.maker = maker
         self.profile = None  # reassigned in FunctionMaker.create
         self.trust_input = False  # If True, we don't check the input parameter
-        self.name = None
+        self.name = name
         self.nodes_with_inner_function = []
         self.output_keys = output_keys
 
+        # See if we have any mutable / borrow inputs
+        # TODO: this only need to be set if there is more then 1 input
+        self._check_for_aliased_inputs = False
+        for i in maker.inputs:
+            # If the input is a shared variable, the memory region is
+            # under Theano control and so we don't need to check if it
+            # is aliased as we never do that.
+            if (isinstance(i, In) and not i.shared and
+                (getattr(i, 'borrow', False) or
+                 getattr(i, 'mutable', False))):
+                self._check_for_aliased_inputs = True
+                break
+
         # We will be popping stuff off this `containers` object.  It is a copy.
         containers = list(self.input_storage)
         finder = {}
@@ -396,7 +414,6 @@ class Function(object):
         # self.input_storage inplace.
         for i, ((input, indices, sinputs), (required, refeed, value)) in \
                 enumerate(zip(self.indices, defaults)):
-            # this is true iff input is not a SymbolicInputKit
             if indices is None:
                 # containers is being used as a stack. Here we pop off
                 # the next one.
@@ -432,41 +449,6 @@ class Function(object):
                     named_inputs.append(input.name)
                 inv_finder[c] = input
                 containers[:1] = []
-            else:
-                # TODO The following code may need to do something to handle
-                # implicit inputs.
-
-                # The input is a SymbolicInputKit, so we take as many
-                # containers as the Kit provides inputs
-                cs = containers[:len(indices)]
-                # distribute does the initialization of the containers
-                input.distribute(value, indices, cs)
-                f = partial(distribute, indices, cs)
-                # Like before, we set a finder entry for the kit. Note that
-                # we are not mapping to a container but to a function which
-                # can reinitialize all the containers
-                finder[i] = f
-                finder[input] = f
-                if input.name not in finder:
-                    finder[input.name] = f
-                else:
-                    finder[input.name] = DUPLICATE
-                # For each input in the kit and its corresponding
-                # container, we put an entry in finder.  This allows
-                # the user to micro-manage elements of the kit if need
-                # be.  All containers inherit the required field and
-                # have their own "provided" counter
-                for c, sin in zip(cs, sinputs):
-                    finder[sin.variable] = c
-                    finder[sin.name] = c
-                    if sin.name not in finder:
-                        finder[sin.name] = c
-                    else:
-                        finder[sin.name] = DUPLICATE
-                    inv_finder[c] = input
-                    c.required = required
-                    c.provided = 0
-                containers[:len(indices)] = []
 
         self.finder = finder
         self.inv_finder = inv_finder
@@ -586,7 +568,8 @@ class Function(object):
 
         Returns
         -------
-        Copied theano.Function
+        theano.Function
+            Copied theano.Function
         """
         # helper function
         def checkSV(sv_ori, sv_rpl):
@@ -647,7 +630,7 @@ class Function(object):
             # Swap SharedVariable in fgraph and In instances
             for index, (i, in_v) in enumerate(zip(ins, fg_cpy.inputs)):
                 # Variables in maker.inputs are defined by user, therefore we
-                # use them to make comparision and do the mapping.
+                # use them to make comparison and do the mapping.
                 # Otherwise we don't touch them.
                 var = maker.inputs[index].variable
 
@@ -699,13 +682,13 @@ class Function(object):
         input_storage = [i.value for i in ins]
         # reinitialize new maker and create new function
         if profile is None:
-            profile = config.profile
+            profile = config.profile or config.print_global_stats
             # profile -> True or False
         if profile is True:
             if name:
                 message = name
             else:
-                message = str(maker.profile.message) + " copy"
+                message = str(profile.message) + " copy"
             profile = theano.compile.profiling.ProfileStats(message=message)
             # profile -> object
         elif type(profile) == str:
@@ -713,7 +696,13 @@ class Function(object):
 
         f_cpy = maker.__class__(inputs=ins, outputs=outs, fgraph=fg_cpy,
                                 mode=maker.mode, profile=profile,
-                                on_unused_input=maker.on_unused_input,
+                                # When removing updates containing variables
+                                # not used in the output function, copy
+                                # generates an unused implicit input.
+                                # We ignore the resulting errors,
+                                # but could change it to 'warn' if this might
+                                # cause problems.
+                                on_unused_input='ignore',
                                 function_builder=maker.function_builder,
                                 # As this is an optimized graph, it
                                 # can contain inplace. DebugMode check
@@ -752,9 +741,47 @@ class Function(object):
         return f_cpy
 
     def __call__(self, *args, **kwargs):
+        """
+        Evaluates value of a function on given arguments.
+
+        Parameters
+        ----------
+        args : list
+            List of inputs to the function. All inputs are required, even when
+            some of them are not necessary to calculate requested subset of
+            outputs.
+
+        kwargs : dict
+            The function inputs can be passed as keyword argument. For this, use
+            the name of the input or the input instance as the key.
+
+            Keyword argument ``output_subset`` is a list of either indices of the
+            function's outputs or the keys belonging to the `output_keys` dict
+            and represent outputs that are requested to be calculated. Regardless
+            of the presence of ``output_subset``, the updates are always calculated
+            and processed. To disable the updates, you should use the ``copy``
+            method with ``delete_updates=True``.
+
+        Returns
+        -------
+        list
+            List of outputs on indices/keys from ``output_subset`` or all of them,
+            if ``output_subset`` is not passed.
+        """
+        def restore_defaults():
+            for i, (required, refeed, value) in enumerate(self.defaults):
+                if refeed:
+                    if isinstance(value, gof.Container):
+                        value = value.storage[0]
+                    self[i] = value
         profile = self.profile
         t0 = time.time()
 
+        output_subset = kwargs.pop('output_subset', None)
+        if output_subset is not None and self.output_keys is not None:
+            output_subset =\
+                [self.output_keys.index(key) for key in output_subset]
+
         # Reinitialize each container's 'provided' counter
         if self.trust_input:
             i = 0
@@ -787,10 +814,22 @@ class Function(object):
 
                     except Exception as e:
                         function_name = "theano function"
+                        argument_name = "argument"
                         if self.name:
-                            function_name += ' with name "' + self.name + '" '
-                        e.args = ("Bad input argument to " + function_name +
-                                  " at index %d(0-based)" % i,) + e.args
+                            function_name += ' with name "' + self.name + '"'
+                        if hasattr(arg, 'name') and arg.name:
+                            argument_name += ' with name "' + arg.name + '"'
+                        where = theano.gof.utils.get_variable_trace_string(
+                            self.maker.inputs[i].variable)
+                        if len(e.args) == 1:
+                            e.args = ("Bad input " + argument_name + " to " +
+                                      function_name + " at index %d (0-based). %s"
+                                      % (i, where) + e.args[0],)
+                        else:
+                            e.args = ("Bad input " + argument_name + " to " +
+                                      function_name + " at index %d (0-based). %s"
+                                      % (i, where),) + e.args
+                        restore_defaults()
                         raise
                 s.provided += 1
                 i += 1
@@ -801,6 +840,7 @@ class Function(object):
                 self[k] = arg
 
         if (not self.trust_input and
+            # The getattr is only needed for old pickle
                 getattr(self, '_check_for_aliased_inputs', True)):
             # Collect aliased inputs among the storage space
             args_share_memory = []
@@ -816,9 +856,9 @@ class Function(object):
                              in args_share_memory[j]],
                             [self.input_storage[k].storage[0] for k
                              in args_share_memory[j]])
-                        if numpy.any([(var.type is i_var.type and
-                                       var.type.may_share_memory(val, i_val))
-                                      for (var, val) in group_j]):
+                        if any([(var.type is i_var.type and
+                                 var.type.may_share_memory(val, i_val))
+                                for (var, val) in group_j]):
 
                             is_aliased = True
                             args_share_memory[j].append(i)
@@ -827,27 +867,30 @@ class Function(object):
                     if not is_aliased:
                         args_share_memory.append([i])
 
-                # Check for groups of more than one argument that share memory
-                for group in args_share_memory:
-                    if len(group) > 1:
-                        # copy all but the first
-                        for idx in group[1:]:
-                            self.input_storage[i].storage[0] = copy.copy(
-                                self.input_storage[i].storage[0])
+            # Check for groups of more than one argument that share memory
+            for group in args_share_memory:
+                if len(group) > 1:
+                    # copy all but the first
+                    for j in group[1:]:
+                        self.input_storage[j].storage[0] = copy.copy(
+                            self.input_storage[j].storage[0])
 
         # Check if inputs are missing, or if inputs were set more than once, or
         # if we tried to provide inputs that are supposed to be implicit.
         if not self.trust_input:
             for c in self.input_storage:
                 if c.required and not c.provided:
+                    restore_defaults()
                     raise TypeError("Missing required input: %s" %
                                     getattr(self.inv_finder[c], 'variable',
                                             self.inv_finder[c]))
                 if c.provided > 1:
+                    restore_defaults()
                     raise TypeError("Multiple values for input: %s" %
                                     getattr(self.inv_finder[c], 'variable',
                                             self.inv_finder[c]))
                 if c.implicit and c.provided > 0:
+                    restore_defaults()
                     raise TypeError(
                         'Tried to provide value for implicit input: %s'
                         % getattr(self.inv_finder[c], 'variable',
@@ -856,8 +899,11 @@ class Function(object):
         # Do the actual work
         t0_fn = time.time()
         try:
-            outputs = self.fn()
+            outputs =\
+                self.fn() if output_subset is None else\
+                self.fn(output_subset=output_subset)
         except Exception:
+            restore_defaults()
             if hasattr(self.fn, 'position_of_error'):
                 # this is a new vm-provided function or c linker
                 # they need this because the exception manipulation
@@ -910,11 +956,7 @@ class Function(object):
             outputs = outputs[:self.n_returned_outputs]
 
         # Put default values back in the storage
-        for i, (required, refeed, value) in enumerate(self.defaults):
-            if refeed:
-                if isinstance(value, gof.Container):
-                    value = value.storage[0]
-                self[i] = value
+        restore_defaults()
         #
         # NOTE: This logic needs to be replicated in
         #       scan.
@@ -922,16 +964,20 @@ class Function(object):
         #
 
         dt_call = time.time() - t0
+        theano.compile.profiling.total_fct_exec_time += dt_call
         self.maker.mode.call_time += dt_call
         if profile:
             profile.fct_callcount += 1
             profile.fct_call_time += dt_call
             if hasattr(self.fn, 'update_profile'):
                 self.fn.update_profile(profile)
-
+            if profile.ignore_first_call:
+                profile.reset()
+                profile.ignore_first_call = False
         if self.return_none:
             return None
-        elif self.unpack_single and len(outputs) == 1:
+        elif self.unpack_single and len(outputs) == 1 and\
+                output_subset is None:
             return outputs[0]
         else:
 
@@ -939,9 +985,16 @@ class Function(object):
 
                 assert len(self.output_keys) == len(outputs)
 
-                return dict(izip(self.output_keys, outputs))
+                if output_subset is None:
+                    return dict(izip(self.output_keys, outputs))
+                else:
+                    return dict((self.output_keys[index], outputs[index])
+                                for index in output_subset)
 
-            return outputs
+            if output_subset is None:
+                return outputs
+            else:
+                return [outputs[i] for i in output_subset]
 
     value = property(
         lambda self: self._value,
@@ -973,6 +1026,15 @@ class Function(object):
         """
         return [i.variable for i in self.maker.inputs if i.implicit]
 
+    def sync_shared(self):
+        if (hasattr(theano, "gpuarray") and
+                theano.gpuarray.pygpu_activated):
+            import pygpu
+            for i in self.maker.fgraph.update_mapping.values():
+                inp = self.input_storage[i]
+                if isinstance(inp.data, pygpu.gpuarray.GpuArray):
+                    inp.data.sync()
+
 
 # pickling/deepcopy support for Function
 def _pickle_Function(f):
@@ -982,16 +1044,8 @@ def _pickle_Function(f):
 
     for (input, indices, inputs), (required, refeed, default) in \
             zip(f.indices, f.defaults):
-        if isinstance(input, SymbolicInputKit):
-            li = len(indices)
-            if not default:
-                input_storage.append(ins[:li])
-            else:
-                input_storage.append(default)
-            ins[:li] = []
-        else:
-            input_storage.append(ins[0])
-            del ins[0]
+        input_storage.append(ins[0])
+        del ins[0]
 
     inputs_data = [x.data for x in f.input_storage]
 
@@ -1002,9 +1056,9 @@ def _pickle_Function(f):
         all_data = input_storage + inputs_data
         for i, d_i in enumerate(all_data):
             for j, d_j in enumerate(all_data):
-                if ((i < j) and isinstance(d_i, numpy.ndarray) and
-                        isinstance(d_j, numpy.ndarray)):
-                    if numpy.may_share_memory(d_i, d_j):
+                if ((i < j) and isinstance(d_i, np.ndarray) and
+                        isinstance(d_j, np.ndarray)):
+                    if np.may_share_memory(d_i, d_j):
                         if f.pickle_aliased_memory_strategy == 'warn':
                             _logger.warning('aliased relationship between '
                                             'Function arguments %s, %s '
@@ -1013,19 +1067,25 @@ def _pickle_Function(f):
                                             (str(d_i), str(d_j)))
                         else:
                             raise AliasedMemoryError(d_i, d_j)
-    rval = (_constructor_Function, (f.maker, input_storage, inputs_data))
+    # The user can override trust_input. Our doc tell that.  We should
+    # not do that anymore and make sure the Maker have all the
+    # information needed.
+    rval = (_constructor_Function,
+            (f.maker, input_storage, inputs_data, f.trust_input))
     return rval
 
 
-def _constructor_Function(maker, input_storage, inputs_data):
+def _constructor_Function(maker, input_storage, inputs_data, trust_input=False):
     if not theano.config.unpickle_function:
         return None
+
     f = maker.create(input_storage, trustme=True)
     assert len(f.input_storage) == len(inputs_data)
     for container, x in zip(f.input_storage, inputs_data):
         assert (container.data is x) or \
-            (isinstance(x, numpy.ndarray) and (container.data == x).all()) or \
+            (isinstance(x, np.ndarray) and (container.data == x).all()) or \
             (container.data == x)
+    f.trust_input = trust_input
     return f
 
 copyreg.pickle(Function, _pickle_Function)
@@ -1034,7 +1094,6 @@ copyreg.pickle(Function, _pickle_Function)
 ###
 # FunctionMaker
 ###
-
 def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
     """
     Insert deepcopy in the fgraph to break aliasing of outputs
@@ -1050,17 +1109,18 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
     # memory contract
 
     # We don't insert deep copy when the output.borrow is True for all
-    # conserned outputs.
+    # concerned outputs.
 
     assert len(wrapped_inputs) == len(fgraph.inputs)
     assert len(wrapped_outputs) == len(fgraph.outputs)
     reason = "insert_deepcopy"
-    updated_fgraph_inputs = [fgraph_i for i, fgraph_i in
-                             zip(wrapped_inputs, fgraph.inputs)
-                             if getattr(i, 'update', False)]
+    updated_fgraph_inputs = set([fgraph_i for i, fgraph_i in
+                                zip(wrapped_inputs, fgraph.inputs)
+                                if getattr(i, 'update', False)])
 
     # We can't use fgraph.inputs as this don't include Constant Value.
     all_graph_inputs = gof.graph.inputs(fgraph.outputs)
+    has_destroyers_attr = hasattr(fgraph, 'has_destroyers')
 
     for i in xrange(len(fgraph.outputs)):
         views_of_output_i = set()
@@ -1089,12 +1149,9 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
                 #    e.g. in-place computations
                 # b) that j'th input is a shared variable that is also
                 #    being updated
-                if (hasattr(fgraph, 'get_destroyers_of') and
-                        fgraph.get_destroyers_of(input_j)):
-                    continue
                 if input_j in updated_fgraph_inputs:
                     continue
-                if input_j in views_of_output_i:
+                if input_j in views_of_output_i and not (has_destroyers_attr and fgraph.has_destroyers([input_j])):
                     # We don't put deep_copy_op if the input and the
                     # output have borrow==True
                     if input_j in fgraph.inputs:
@@ -1154,12 +1211,15 @@ class FunctionMaker(object):
         - 'warn': log a warning
         - 'ignore': do not do anything
         - None: Use the value in the Theano flags on_unused_input.
+    name : str
+        An optional name for this function. If used, the profile mode will
+        print the time spent in this function.
 
     """
 
     @staticmethod
     def wrap_in(input):
-        if isinstance(input, (SymbolicInput, SymbolicInputKit)):
+        if isinstance(input, (SymbolicInput)):
             return input
         elif isinstance(input, gof.Variable):
             # r -> SymbolicInput(variable=r)
@@ -1183,9 +1243,10 @@ class FunctionMaker(object):
         # instances in inputs.  For SymbolicInput, this returns None
         # as the list of indices and a list with just the
         # SymbolicInput.
-        if isinstance(sinput, SymbolicInputKit):
-            return sinput.complete(rinputs)
-        elif isinstance(sinput, SymbolicInput):
+        # if isinstance(sinput, SymbolicInputKit):
+        #    return sinput.complete(rinputs)
+        # elif isinstance(sinput, SymbolicInput):
+        if isinstance(sinput, SymbolicInput):
             return [None, [sinput]]
 
     @staticmethod
@@ -1367,20 +1428,19 @@ class FunctionMaker(object):
     def __init__(self, inputs, outputs,
                  mode=None, accept_inplace=False, function_builder=Function,
                  profile=None, on_unused_input=None, fgraph=None,
-                 output_keys=None):
+                 output_keys=None, name=None):
+        # Save the provided mode, not the instanciated mode.
+        # The instanciated mode don't pickle and if we unpickle a Theano
+        # function and it get re-compiled, we want the current optimizer to be
+        # used, not the optimizer when it was saved.
+        self.mode = mode
         mode = theano.compile.mode.get_mode(mode)
 
-        # figure out which profile object to use (if any)
-        # to help with forward-porting ProfileMode,
-        # we allow ProfileMode to provide a ProfileStats object
-        # using this somewhat awkward mechanism.
-        mode_profile = getattr(mode, 'profile', None)
-        if (profile is not None and
-                profile is not False and
-                mode_profile is not None):
+        # Assert old way of working isn't used
+        if getattr(mode, 'profile', None):
             raise TypeError(
-                'profile passed via both "mode" and "profile" arguments')
-        self.profile = profile = profile or mode_profile
+                "profile passed via 'mode'. This isn't supported anymore")
+        self.profile = profile
         if profile:
             # This is very important:
             # 1) We preload the cache here to don't have its timming
@@ -1392,7 +1452,6 @@ class FunctionMaker(object):
             theano.gof.cc.get_module_cache().refresh()
         # Handle the case where inputs and/or outputs is a single
         # Variable (not in a list)
-        self.orig_outputs = outputs
         unpack_single = False
         return_none = False
         if outputs is None:
@@ -1445,9 +1504,13 @@ class FunctionMaker(object):
                 # optimize the fgraph
                 theano.config.compute_test_value = \
                     theano.config.compute_test_value_opt
-                theano.config.traceback.limit = 0
+                theano.config.traceback.limit = theano.config.traceback.compile_limit
                 start_optimizer = time.time()
 
+                # In case there is an error during optimization.
+                optimizer_profile = None
+                opt_time = None
+
                 # now optimize the graph
                 if theano.config.cache_optimizations:
                     optimizer_profile = self.optimize_graph_with_cache(
@@ -1457,15 +1520,6 @@ class FunctionMaker(object):
 
                 end_optimizer = time.time()
                 opt_time = end_optimizer - start_optimizer
-                if profile:
-                    profile.optimizer_time += opt_time
-                    if theano.config.profile_optimizer:
-                        profile.optimizer_profile = (optimizer,
-                                                     optimizer_profile)
-                elif theano.config.profile_optimizer:
-                    warnings.warn((
-                        "config.profile_optimizer requires config.profile to "
-                        " be set to True as well"), stacklevel=3)
                 _logger.debug('Optimizing took %f seconds', opt_time)
 
                 # Add deep copy to respect the memory interface
@@ -1474,6 +1528,25 @@ class FunctionMaker(object):
                 theano.config.compute_test_value = compute_test_value_orig
                 theano.config.traceback.limit = limit_orig
 
+                # If the optimizer got interrupted
+                if opt_time is None:
+                    end_optimizer = time.time()
+                    opt_time = end_optimizer - start_optimizer
+                theano.compile.profiling.total_graph_opt_time += opt_time
+                if profile:
+                    if (optimizer_profile is None and
+                            hasattr(optimizer, 'pre_profile')):
+                        optimizer_profile = optimizer.pre_profile
+                    profile.optimizer_time += opt_time
+                    if theano.config.profile_optimizer:
+                        profile.optimizer_profile = (optimizer,
+                                                     optimizer_profile)
+                # IF False, if mean the profile for that function was explicitly disabled
+                elif theano.config.profile_optimizer and profile is not False:
+                    warnings.warn((
+                        "config.profile_optimizer requires config.profile to "
+                        " be set to True as well"), stacklevel=3)
+
         # initialize the linker
         if not hasattr(linker, 'accept'):
             raise ValueError("'linker' parameter of FunctionMaker should be "
@@ -1489,26 +1562,27 @@ class FunctionMaker(object):
                      if not spec.borrow]
         if no_borrow:
             self.linker = linker.accept(
-                fgraph, no_recycling=infer_reuse_pattern(fgraph, no_borrow))
+                fgraph, no_recycling=infer_reuse_pattern(fgraph, no_borrow),
+                profile=profile)
         else:
-            self.linker = linker.accept(fgraph)
+            self.linker = linker.accept(fgraph, profile=profile)
 
         if hasattr(linker, 'accept_var_updates'):
             # hacky thing so VMLinker knows about updates
             self.linker.accept_var_updates(
                 fgraph_updated_vars(fgraph, inputs))
-
+        fgraph.name = name
         self.indices = indices
         self.inputs = inputs
         self.expanded_inputs = inputs
         self.outputs = outputs
         self.unpack_single = unpack_single
         self.return_none = return_none
-        self.mode = mode
         self.accept_inplace = accept_inplace
         self.function_builder = function_builder
         self.on_unused_input = on_unused_input  # Used for the pickling/copy
         self.output_keys = output_keys
+        self.name = name
 
         self.required = [(i.value is None) for i in self.inputs]
         self.refeed = [
@@ -1636,7 +1710,7 @@ class FunctionMaker(object):
         start_import_time = theano.gof.cmodule.import_time
         limit_orig = theano.config.traceback.limit
         try:
-            theano.config.traceback.limit = 0
+            theano.config.traceback.limit = theano.config.traceback.compile_limit
             _fn, _i, _o = self.linker.make_thunk(
                 input_storage=input_storage_lists, storage_map=storage_map)
         finally:
@@ -1645,6 +1719,7 @@ class FunctionMaker(object):
         end_linker = time.time()
 
         linker_time = end_linker - start_linker
+        theano.compile.profiling.total_time_linker += linker_time
         _logger.debug('Linker took %f seconds', linker_time)
         if self.profile:
             self.profile.linker_time += linker_time
@@ -1654,25 +1729,16 @@ class FunctionMaker(object):
 
         fn = self.function_builder(_fn, _i, _o, self.indices, self.outputs,
                                    defaults, self.unpack_single,
-                                   self.return_none, self.output_keys, self)
+                                   self.return_none, self.output_keys, self,
+                                   name=self.name)
+
         fn.profile = self.profile
         return fn
 
 
-def _pickle_FunctionMaker(self):
-    kwargs = dict(
-        inputs=self.inputs,
-        outputs=self.orig_outputs,
-        fgraph=self.fgraph,
-        mode=self.mode,
-        accept_inplace=self.accept_inplace,
-        function_builder=self.function_builder,
-        profile=self.profile,
-        on_unused_input=self.on_unused_input)
-    return (_constructor_FunctionMaker, (kwargs,))
-
-
 def _constructor_FunctionMaker(kwargs):
+    # Needed for old pickle
+    # Old pickle have at least the problem that output_keys where not saved.
     if theano.config.unpickle_function:
         if theano.config.reoptimize_unpickled_function:
             del kwargs['fgraph']
@@ -1680,8 +1746,6 @@ def _constructor_FunctionMaker(kwargs):
     else:
         return None
 
-copyreg.pickle(FunctionMaker, _pickle_FunctionMaker)
-
 __checkers = []
 
 
@@ -1715,8 +1779,8 @@ def orig_function(inputs, outputs, mode=None, accept_inplace=False,
         Default of None means to use `config.mode` (see below for descriptive
         string list).
     name : str
-        An optional name for this fct. If used, the profile mode will print the
-        time spent in this fct.
+        An optional name for this function. If used, the profile mode will print the
+        time spent in this function.
     accept_inplace : bool
         True iff the graph can contain inplace operations prior to the
         optimization phase (default is False).
@@ -1736,9 +1800,6 @@ def orig_function(inputs, outputs, mode=None, accept_inplace=False,
 
     - FAST_COMPILE (minimal optimization)
 
-    - ProfileMode(deprecated): allow to print a profile mode with
-      mode.print_summary
-
     - DebugMode: verify many internal conditions that are normally assumed
       (slow)
 
@@ -1765,24 +1826,26 @@ def orig_function(inputs, outputs, mode=None, accept_inplace=False,
 
     if isinstance(mode, (list, tuple)):  # "mode comparison" semantics
         raise Exception("We do not support the passing of multiple modes")
-    else:
+    fn = None
+    try:
         Maker = getattr(mode, 'function_maker', FunctionMaker)
-        fn = Maker(inputs,
-                   outputs,
-                   mode,
-                   accept_inplace=accept_inplace,
-                   profile=profile,
-                   on_unused_input=on_unused_input,
-                   output_keys=output_keys).create(
-            defaults)
+        m = Maker(inputs,
+                  outputs,
+                  mode,
+                  accept_inplace=accept_inplace,
+                  profile=profile,
+                  on_unused_input=on_unused_input,
+                  output_keys=output_keys,
+                  name=name)
+        with theano.change_flags(compute_test_value="off"):
+            fn = m.create(defaults)
+    finally:
+        t2 = time.time()
+        if fn and profile:
+            profile.compile_time += t2 - t1
+            # TODO: append
+            profile.nb_nodes = len(fn.maker.fgraph.apply_nodes)
 
-    t2 = time.time()
-    if profile:
-        profile.compile_time += t2 - t1
-        profile.nb_nodes = len(fn.maker.fgraph.apply_nodes)
-
-    fn.name = name
-    fn.maker.fgraph.name = name
     return fn
 
 
@@ -1807,7 +1870,7 @@ def convert_function_input(input):
       `In`(r, name=name, value=val, update=up, autoname=True)
 
     """
-    if isinstance(input, (SymbolicInput, SymbolicInputKit)):
+    if isinstance(input, SymbolicInput):
         return input
     elif isinstance(input, gof.Constant):
         raise TypeError('A Constant instance is not a legal function input',
@@ -1836,7 +1899,7 @@ def convert_function_input(input):
             else:
                 raise TypeError("Invalid input syntax: %s (check "
                                 "documentation or use an In instance)" % orig)
-        elif isinstance(input[0], (SymbolicInput, SymbolicInputKit)):
+        elif isinstance(input[0], SymbolicInput):
             if len(input) == 1:
                 return input[0]
             elif len(input) == 2:
