@@ -3,7 +3,7 @@ Defines Linkers that deal with C implementations.
 
 """
 
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 
 # Python imports
 from copy import copy
@@ -11,7 +11,7 @@ import os
 import sys
 import logging
 
-import numpy
+import numpy as np
 
 import theano
 from theano import config
@@ -61,8 +61,6 @@ def get_persistent_module_cache():
 
 class CodeBlock:
     """
-    WRITEME
-
     Represents a computation unit composed of declare, behavior, and cleanup.
 
     The constructor initializes a L{CodeBlock} with templatized declare,
@@ -96,28 +94,48 @@ class CodeBlock:
                         "\ndouble __DUMMY_%(id)i;\n" % sub)  # % sub
 
 
-def failure_code(sub):
+def failure_code(sub, use_goto=True):
     """
     Code contained in sub['fail'], usually substituted for %(fail)s.
 
     It sets information about current error, then goto the code
     actually handling the failure, which is defined in struct_gen().
 
+    Parameters
+    ----------
+    sub: dict
+        Contains other code snippets that can be substituted,
+        in particular 'failure_var' and 'id'.
+    use_goto: bool, True by default
+        Include a "goto" statement to the failure label.
+        Passing False is sometimes required, in which cases we have to
+        be careful to avoid executing incorrect code.
+
     """
+    if use_goto:
+        goto_statement = 'goto __label_%(id)i;' % sub
+    else:
+        goto_statement = ''
     return '''{
-        %(failure_var)s = %(id)s;
+        %(failure_var)s = %(id)i;
         if (!PyErr_Occurred()) {
             PyErr_SetString(PyExc_RuntimeError,
                 "Unexpected error in an Op's C code. "
                 "No Python exception was set.");
-            }
-        goto __label_%(id)i;}''' % sub
+        }
+        %(goto_statement)s}''' % dict(sub, goto_statement=goto_statement)
 
 
 def failure_code_init(sub):
     """
     Code for failure in the struct init.
 
+    Parameters:
+    ----------
+    sub
+      Dictionary used to template the struct.
+      * failure_var -> must contain a variable name to use for
+      the failure code.
     """
     return '''{
         if (!PyErr_Occurred()) {
@@ -131,10 +149,10 @@ def failure_code_init(sub):
 
 def code_gen(blocks):
     """
-    WRITEME
-
     From a list of L{CodeBlock} instances, returns a string
-    that executes them all in sequence. eg for C{(decl1, task1,
+    that executes them all in sequence.
+
+    Eg for C{(decl1, task1,
     cleanup1)} and C{(decl2, task2, cleanup2)} the returned string
     will be of the form:
 
@@ -149,6 +167,12 @@ def code_gen(blocks):
          cleanup1
         }
 
+    Parameters:
+    ----------
+    blocks
+         List of CodeBlock instances such that
+         * declarations, behavior and cleanup are in the run()
+         method of the struct
     """
     decl = ""
     head = ""
@@ -162,8 +186,6 @@ def code_gen(blocks):
 
 def struct_gen(args, struct_builders, blocks, sub):
     """
-    WRITEME
-
     Generates a struct conforming to the following specifications:
 
     Parameters
@@ -277,7 +299,9 @@ def struct_gen(args, struct_builders, blocks, sub):
             // now I am tired of chasing segfaults because
             // initialization code had an error and some pointer has
             // a junk value.
+            #ifndef THEANO_DONT_MEMSET_STRUCT
             memset(this, 0, sizeof(*this));
+            #endif
         }
         ~%(name)s(void) {
             cleanup();
@@ -399,7 +423,7 @@ def get_c_extract_out(r, name, sub):
     # `c_extract_out` is used to extract an output variable from
     # the compute map, to be used as pre-allocated memory for `r`
     # before its value gets computed.
-    # If the node producing `r` has `check_inputs=True`, it may
+    # If the node producing `r` has `check_input=True`, it may
     # also perform type checks on the initial value of the output,
     # so we need to pass `check_input=True` to `c_extract_out`.
     # However, that code is not used by potential clients of `r`,
@@ -453,7 +477,7 @@ def get_c_sync(r, name, sub):
 
 def apply_policy(policy, r, name, sub):
     """
-    WRITEME
+    Apply the list of policies to name.r,sub
 
     Parameters
     ----------
@@ -478,7 +502,7 @@ def apply_policy(policy, r, name, sub):
 
 def struct_variable_codeblocks(variable, policies, id, symbol_table, sub):
     """
-    WRITEME
+    Update "sub" dict and create two codeblocks with different failure modes
 
     Parameters
     ----------
@@ -525,8 +549,6 @@ def struct_variable_codeblocks(variable, policies, id, symbol_table, sub):
 
 class CLinker(link.Linker):
     """
-    WRITEME
-
     Creates C code for an fgraph, compiles it and returns callables
     through make_thunk and make_function that make use of the compiled
     code.
@@ -542,16 +564,17 @@ class CLinker(link.Linker):
         if schedule:
             self.schedule = schedule
 
-    def accept(self, fgraph, no_recycling=None):
+    def accept(self, fgraph, no_recycling=None, profile=None):
         """
-        WRITEME
+        Associate linker with fgraph
 
         """
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
             # A linker can be tied to only one FunctionGraph.
-            return type(self)(self.schedule).accept(fgraph, no_recycling)
+            return type(self)(self.schedule).accept(
+                fgraph, no_recycling, profile)
         self.fgraph = fgraph
         self.fetch_variables()
         self.no_recycling = no_recycling
@@ -559,8 +582,6 @@ class CLinker(link.Linker):
 
     def fetch_variables(self):
         """
-        WRITEME
-
         Fills the inputs, outputs, variables, orphans, temps and node_order
         fields.
 
@@ -617,8 +638,6 @@ class CLinker(link.Linker):
 
     def code_gen(self):
         """
-        WRITEME
-
         Generates code for a struct that does the computation of the fgraph and
         stores it in the struct_code field of the instance.
 
@@ -715,7 +734,7 @@ class CLinker(link.Linker):
                               [get_c_declare, get_c_extract_out,
                                   (get_c_sync, get_c_cleanup)]]
             else:
-                raise Exception("what the fuck")
+                raise Exception("this shouldn't be possible, please report this exception")
 
             builder, block = struct_variable_codeblocks(variable, policy,
                                                         id, symbol, sub)
@@ -748,8 +767,6 @@ class CLinker(link.Linker):
             # This ensures that, when defining functions in support code,
             # we cannot have two different functions, in different modules,
             # that have the same name.
-            # It was problematic, in particular, on Mac OS X (10.6 and 10.7)
-            # when defining CUDA kernels (with Cuda 4.2 and 5.0). See gh-1172.
             name = "node_<<<<HASH_PLACEHOLDER>>>>_%i" % node_num
             isyms = [symbol[r] for r in node.inputs]
             osyms = [symbol[r] for r in node.outputs]
@@ -890,29 +907,32 @@ class CLinker(link.Linker):
 
     def support_code(self):
         """
-        WRITEME
-
         Returns a list of support code strings that are needed by
-        one or more Variables or Ops. The support code from Variables is
-        added before the support code from Ops.
-
-        This might contain duplicates.
-
+        one or more Variables or Ops.
+        The support code from Variables is added before the support code from Ops.This might contain duplicates.
         """
         ret = []
+        if config.cmodule.debug:
+            ret.append("""
+            #ifndef DEBUG
+            #define DEBUG
+            #endif
+            """)
         # generic support code
         for x in [y.type for y in self.variables] + [
                 y.op for y in self.node_order]:
             try:
-                ret.append(x.c_support_code())
+                support_code = x.c_support_code()
+                if isinstance(support_code, list):
+                    ret.extend(support_code)
+                else:
+                    ret.append(support_code)
             except utils.MethodNotDefined:
                 pass
         return ret
 
     def compile_args(self):
         """
-        WRITEME
-
         Returns a list of compile args that are needed by one
         or more Variables or Ops.
 
@@ -971,8 +991,6 @@ class CLinker(link.Linker):
 
     def headers(self):
         """
-        WRITEME
-
         Returns a list of headers that are needed by one
         or more Types or Ops.
 
@@ -1032,8 +1050,6 @@ class CLinker(link.Linker):
 
     def header_dirs(self):
         """
-        WRITEME
-
         Returns a list of lib directories that are needed by one
         or more Types or Ops.
 
@@ -1051,12 +1067,11 @@ class CLinker(link.Linker):
                     ret += x.c_header_dirs()
             except utils.MethodNotDefined:
                 pass
-        return utils.uniq(ret)
+        # filter out empty strings/None
+        return [r for r in utils.uniq(ret) if r]
 
     def libraries(self):
         """
-        WRITEME
-
         Returns a list of libraries that are needed by one
         or more Types or Ops.
 
@@ -1078,8 +1093,6 @@ class CLinker(link.Linker):
 
     def lib_dirs(self):
         """
-        WRITEME
-
         Returns a list of lib directories that are needed by one
         or more Types or Ops.
 
@@ -1097,11 +1110,12 @@ class CLinker(link.Linker):
                     ret += x.c_lib_dirs()
             except utils.MethodNotDefined:
                 pass
-        return utils.uniq(ret)
+        # filter out empty strings/None
+        return [r for r in utils.uniq(ret) if r]
 
     def __compile__(self, input_storage=None, output_storage=None,
                     storage_map=None, keep_lock=False):
-        """WRITEME
+        """
         Compiles this linker's fgraph.
 
         Parameters
@@ -1136,12 +1150,13 @@ class CLinker(link.Linker):
                 output_storage.append(map[variable])
         input_storage = tuple(input_storage)
         output_storage = tuple(output_storage)
-        thunk = self.cthunk_factory(error_storage,
-                                    input_storage,
-                                    output_storage,
-                                    storage_map,
-                                    keep_lock=keep_lock)
+        thunk, module = self.cthunk_factory(error_storage,
+                                            input_storage,
+                                            output_storage,
+                                            storage_map,
+                                            keep_lock=keep_lock)
         return (thunk,
+                module,
                 [link.Container(input, storage) for input, storage in
                  izip(self.fgraph.inputs, input_storage)],
                 [link.Container(output, storage, True) for output, storage in
@@ -1166,7 +1181,7 @@ class CLinker(link.Linker):
 
     def make_thunk(self, input_storage=None, output_storage=None,
                    storage_map=None, keep_lock=False):
-        """WRITEME
+        """
         Compiles this linker's fgraph and returns a function to perform the
         computations, as well as lists of storage cells for both the inputs
         and outputs.
@@ -1183,8 +1198,10 @@ class CLinker(link.Linker):
             be allocated.
         storage_map: dict that map variables to storages.
             This is used when you need to customize the storage of
-            this thunk.
-
+            this thunk
+        keep_lock:
+            If True, we won't release the lock on the compiledir
+            at the end of this function call.
         Returns: thunk, input_storage, output_storage
 
         The return values can be used as follows:
@@ -1195,11 +1212,11 @@ class CLinker(link.Linker):
           first_output = ostor[0].data
         """
         init_tasks, tasks = self.get_init_tasks()
-        cthunk, in_storage, out_storage, error_storage = self.__compile__(
+        cthunk, module, in_storage, out_storage, error_storage = self.__compile__(
             input_storage, output_storage, storage_map,
             keep_lock=keep_lock)
 
-        res = _CThunk(cthunk, init_tasks, tasks, error_storage)
+        res = _CThunk(cthunk, init_tasks, tasks, error_storage, module)
         res.nodes = self.node_order
         return res, in_storage, out_storage
 
@@ -1214,25 +1231,23 @@ class CLinker(link.Linker):
         The signature has the following form:
         {{{
             'CLinker.cmodule_key', compilation args, libraries,
-            header_dirs, numpy ABI version, config md5,
+            header_dirs, numpy ABI version, config hash,
             (op0, input_signature0, output_signature0),
             (op1, input_signature1, output_signature1),
             ...
             (opK, input_signatureK, output_signatureK),
         }}}
 
+        Note that config hash now uses sha256, and not md5.
+
         The signature is a tuple, some elements of which are sub-tuples.
 
         The outer tuple has a brief header, containing the compilation options
-        passed to the compiler, the libraries to link against, an md5 hash
+        passed to the compiler, the libraries to link against, a sha256 hash
         of theano.config (for all config options where "in_c_key" is True).
         It is followed by elements for every node in the topological ordering
         of `self.fgraph`.
 
-        If the Op of any Apply in the FunctionGraph does not have
-        c_code_cache_ok()==True, then this function raises a KeyError
-        exception.
-
         Input Signature
         ---------------
 
@@ -1287,7 +1302,7 @@ class CLinker(link.Linker):
 
     def cmodule_key_variables(self, inputs, outputs, no_recycling,
                               compile_args=None, libraries=None,
-                              header_dirs=None, insert_config_md5=True,
+                              header_dirs=None, insert_config_hash=True,
                               c_compiler=None):
 
         # Assemble a dummy fgraph using the provided inputs and outputs. It is
@@ -1310,11 +1325,11 @@ class CLinker(link.Linker):
 
         fgraph = FakeFunctionGraph(inputs, outputs)
         return self.cmodule_key_(fgraph, no_recycling, compile_args,
-                                 libraries, header_dirs, insert_config_md5,
+                                 libraries, header_dirs, insert_config_hash,
                                  c_compiler)
 
     def cmodule_key_(self, fgraph, no_recycling, compile_args=None,
-                     libraries=None, header_dirs=None, insert_config_md5=True,
+                     libraries=None, header_dirs=None, insert_config_hash=True,
                      c_compiler=None):
         """
         Do the actual computation of cmodule_key in a static method
@@ -1336,7 +1351,7 @@ class CLinker(link.Linker):
         constant_ids = dict()
         op_pos = {}  # Apply -> topological position
 
-        # First we put the header, compile_args, library names and config md5
+        # First we put the header, compile_args, library names and config hash
         # into the signature.
         sig = ['CLinker.cmodule_key']  # will be cast to tuple on return
         if compile_args is not None:
@@ -1360,8 +1375,12 @@ class CLinker(link.Linker):
 
         # We must always add the numpy ABI version here as
         # DynamicModule always add the include <numpy/arrayobject.h>
+        if np.lib.NumpyVersion(np.__version__) < '1.16.0a':
+            ndarray_c_version = np.core.multiarray._get_ndarray_c_version()
+        else:
+            ndarray_c_version = np.core._multiarray_umath._get_ndarray_c_version()
         sig.append('NPY_ABI_VERSION=0x%X' %
-                   numpy.core.multiarray._get_ndarray_c_version())
+                   ndarray_c_version)
         if c_compiler:
             sig.append('c_compiler_str=' + c_compiler.version_str())
 
@@ -1369,8 +1388,11 @@ class CLinker(link.Linker):
         # parameters from the rest of the key. If you want to add more key
         # elements, they should be before this md5 hash if and only if they
         # can lead to a different compiled file with the same source code.
-        if insert_config_md5:
-            sig.append('md5:' + theano.configparser.get_config_md5())
+
+        # NOTE: config md5 is not using md5 hash, but sha256 instead. Function
+        # string instances of md5 will be updated at a later release.
+        if insert_config_hash:
+            sig.append('md5:' + theano.configparser.get_config_hash())
         else:
             sig.append('md5: <omitted>')
 
@@ -1426,11 +1448,10 @@ class CLinker(link.Linker):
 
         version = []
         for node_pos, node in enumerate(order):
-            try:
-                # Pure Ops do not have a c_code_cache_version_apply ...
+            if hasattr(node.op, 'c_code_cache_version_apply'):
                 version.append(node.op.c_code_cache_version_apply(node))
-            except AttributeError:
-                pass
+            if hasattr(node.op, '__props__'):
+                version.append(node.op.__props__)
             for i in node.inputs:
                 version.append(i.type.c_code_cache_version())
             for o in node.outputs:
@@ -1572,7 +1593,12 @@ class CLinker(link.Linker):
 
     def cthunk_factory(self, error_storage, in_storage, out_storage,
                        storage_map=None, keep_lock=False):
-        """WRITEME
+        """
+        Returns a thunk that points to an instance of a C struct that
+        can carry on the computation of this linker's fgraph
+
+        Parameters:
+        ----------
         error_storage -> list of length 3
         in_storage -> list of lists of length 1, one per input
         out_storage -> list of lists of length 1, one per output
@@ -1591,6 +1617,9 @@ class CLinker(link.Linker):
             # If we can't get a key, then forget the cache mechanism.
             module = self.compile_cmodule()
         else:
+            # Set compute_map as None as clinker do not support lazy evaluation
+            for node in self.node_order:
+                node.op.prepare_node(node, storage_map, None, 'c')
             module = get_module_cache().module_from_key(
                 key=key, lnk=self, keep_lock=keep_lock)
 
@@ -1610,8 +1639,7 @@ class CLinker(link.Linker):
 
         ret = module.instantiate(error_storage,
                                  *(in_storage + out_storage + orphd))
-
-        return ret
+        return ret, module
 
     def instantiate_code(self, n_args):
         code = StringIO()
@@ -1656,10 +1684,13 @@ class _CThunk(object):
         WRITEME
     error_storage
         WRITEME
+    module
+        The module that was used to compile this cthunk.
+        Mostly only useful for tests.
 
     """
 
-    def __init__(self, cthunk, init_tasks, tasks, error_storage):
+    def __init__(self, cthunk, init_tasks, tasks, error_storage, module):
         global run_cthunk
         if run_cthunk is None:
             # Lazy import to avoid compilation when importing theano.
@@ -1668,6 +1699,7 @@ class _CThunk(object):
         self.init_tasks = init_tasks
         self.tasks = tasks
         self.error_storage = error_storage
+        self.module = module
 
     def find_task(self, failure_code):
         """
@@ -1709,8 +1741,6 @@ class _CThunk(object):
 
 class OpWiseCLinker(link.LocalLinker):
     """
-    WRITEME
-
     Uses CLinker on the individual Ops that comprise an fgraph and loops
     over them in Python. The variable is slower than a compiled version of
     the whole fgraph, but saves on compilation time because small changes
@@ -1749,7 +1779,10 @@ class OpWiseCLinker(link.LocalLinker):
         if schedule:
             self.schedule = schedule
 
-    def accept(self, fgraph, no_recycling=None):
+    def accept(self, fgraph, no_recycling=None, profile=None):
+        """
+        Associate linker with fgraph
+        """
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
@@ -1759,7 +1792,7 @@ class OpWiseCLinker(link.LocalLinker):
                 allow_gc=self.allow_gc,
                 nice_errors=self.nice_errors,
                 schedule=self.schedule,
-            ).accept(fgraph, no_recycling)
+            ).accept(fgraph, no_recycling, profile)
         self.fgraph = fgraph
         self.no_recycling = no_recycling
         return self
@@ -1768,7 +1801,7 @@ class OpWiseCLinker(link.LocalLinker):
                  storage_map=None):
 
         # The lock will be acquired when we compile the first
-        # C code. We will keep the lock untill all the function
+        # C code. We will keep the lock until all the function
         # compilation will be finished. This allow to don't
         # require the lock when all c code are already compiled!
         orig_n_lock = getattr(get_lock, "n_lock", 0)
@@ -1792,24 +1825,14 @@ class OpWiseCLinker(link.LocalLinker):
 
             thunks = []
             for node in order:
-                # Maker sure we use the C version of the code whenever
-                # possible
-                # There are ops that don't have _op_use_c_code property
-                # for example ifelse (or any ops that come with their own
-                # make_thunk
-                old_value = getattr(node.op, '_op_use_c_code', False)
-                try:
-                    if theano.config.cxx:
-                        node.op._op_use_c_code = True
-                    thunks += [node.op.make_thunk(node,
-                                                  storage_map,
-                                                  compute_map,
-                                                  no_recycling)]
-                    thunks[-1].inputs = [storage_map[v] for v in node.inputs]
-                    thunks[-1].outputs = [storage_map[v] for v in node.outputs]
-
-                finally:
-                    node.op._op_use_c_code = old_value
+                # make_thunk will try by default C code, otherwise
+                # it fall back to python.
+                thunks += [node.op.make_thunk(node,
+                                              storage_map,
+                                              compute_map,
+                                              no_recycling)]
+                thunks[-1].inputs = [storage_map[v] for v in node.inputs]
+                thunks[-1].outputs = [storage_map[v] for v in node.outputs]
 
             for node in order:
                 if self.allow_gc:
@@ -1850,11 +1873,14 @@ class OpWiseCLinker(link.LocalLinker):
 
 def _default_checker(x, y):
     """
-    WRITEME
-
     Default checker for DualLinker. This checks that the
     variables contain the same data using ==.
 
+
+    Parameters:
+    ----------
+    x,y
+        the variables to compare data
     """
     if x[0] != y[0]:
         raise Exception("Output mismatch.",
@@ -1863,8 +1889,6 @@ def _default_checker(x, y):
 
 class DualLinker(link.Linker):
     """
-    WRITEME
-
     Runs the fgraph in parallel using PerformLinker and CLinker.
 
     The thunk/function produced by DualLinker uses PerformLinker as the
@@ -1905,18 +1929,24 @@ class DualLinker(link.Linker):
         if schedule:
             self.schedule = schedule
 
-    def accept(self, fgraph, no_recycling=None):
+    def accept(self, fgraph, no_recycling=None, profile=None):
+        """
+        Update/tie self with fgraph
+        """
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
             return type(self)(self.checker, self.schedule).accept(
-                fgraph, no_recycling)
+                fgraph, no_recycling, profile)
         self.fgraph = fgraph
         self.no_recycling = no_recycling
         return self
 
     def make_thunk(self, **kwargs):
-
+        """
+        Compiles this linker's fgraph and returns a function to perform the
+        computations
+        """
         fgraph = self.fgraph
         no_recycling = self.no_recycling
 
