@@ -2,12 +2,12 @@
 WRITEME
 
 """
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 from copy import copy, deepcopy
 from sys import getsizeof
 import sys
 import traceback
-import numpy
+import numpy as np
 
 import theano
 from theano.compat import izip
@@ -52,16 +52,24 @@ def log_thunk_trace(value, f=sys.stderr):
 
 def thunk_hook(type, value, trace):
     """
-    WRITEME
-
     This function is meant to replace excepthook and do some
     special work if the exception value has a __thunk_trace__
-    field. In that case, it retrieves the field, which should
+    field.
+    In that case, it retrieves the field, which should
     contain a trace as returned by L{traceback.extract_stack},
     and prints it out on L{stderr}.
 
     The normal excepthook is then called.
 
+    Parameters:
+    ----------
+    type
+        Exception class
+    value
+        Exception instance
+    trace
+        Traceback object
+
     Notes
     -----
     This hook replaced by nosetests, so it does not run in nose tests.
@@ -157,6 +165,9 @@ def raise_with_op(node, thunk=None, exc_info=None, storage_map=None):
         detailed_err_msg += ("Inputs shapes: %s" % shapes +
                              "\nInputs strides: %s" % strides +
                              "\nInputs values: %s" % scalar_values)
+        if theano.config.exception_verbosity == 'high':
+            detailed_err_msg += "\nInputs type_num: %s" % str(
+                [getattr(getattr(i[0], 'dtype', ''), 'num', '') for i in thunk.inputs])
         if hasattr(node.op, '__input_name__'):
             detailed_err_msg += "\nInputs name: %s\n" % str(node.op.__input_name__)
 
@@ -168,7 +179,7 @@ def raise_with_op(node, thunk=None, exc_info=None, storage_map=None):
 
     # Print node backtraces
     tr = getattr(node.outputs[0].tag, 'trace', [])
-    if type(tr) is list and len(tr) > 0:
+    if isinstance(tr, list) and len(tr) > 0:
         detailed_err_msg += "\nBacktrace when the node is created(use Theano flag traceback.limit=N to make it longer):\n"
 
         # Print separate message for each element in the list of batcktraces
@@ -225,11 +236,11 @@ def raise_with_op(node, thunk=None, exc_info=None, storage_map=None):
             # storage_map_item[3]: bytes
             if hasattr(storage_map[k][0], 'dtype'):
                 dtype = storage_map[k][0].dtype
-                storage_map_item.append(numpy.dtype(dtype).itemsize)
+                storage_map_item.append(np.dtype(dtype).itemsize)
                 if shapeinfo is None:
                     storage_map_item.append(-1)
                 else:
-                    sz = numpy.dtype(dtype).itemsize * numpy.prod(shapeinfo)
+                    sz = np.dtype(dtype).itemsize * np.prod(shapeinfo)
                     storage_map_item.append(sz)
                     total_size += sz
                     if not k.owner:
@@ -489,7 +500,7 @@ class Container(object):
             deepcopy(self.allow_downcast, memo=memo),
             deepcopy(self.name, memo=memo),
         )
-        # Work around NumPy deepcopy of ndarray with 0 dimention that
+        # Work around NumPy deepcopy of ndarray with 0 dimension that
         # don't return an ndarray.
         if (r.storage[0] is not None and
                 not self.type.is_valid_value(r.storage[0])):
@@ -677,8 +688,6 @@ def streamline(fgraph, thunks, order, post_thunk_old_storage=None,
 
 class LocalLinker(Linker):
     """
-    WRITEME
-
     Useful base class for L{Linker}s which keep all nodes in the graph, and run
     a thunk associated with each node.
 
@@ -704,7 +713,7 @@ class LocalLinker(Linker):
 
 def gc_helper(node_list):
     """
-
+    Return the set of Variable instances which are computed by node_list.
     Parameters
     ----------
     node_list
@@ -740,8 +749,6 @@ def gc_helper(node_list):
 
 class PerformLinker(LocalLinker):
     """
-    WRITEME
-
     Basic L{Linker} subclass that calls the perform method on each L{Op} in
     the L{FunctionGraph} in the order given by L{Linker.schedule}.
 
@@ -755,14 +762,13 @@ class PerformLinker(LocalLinker):
         if schedule:
             self.schedule = schedule
 
-    def accept(self, fgraph, no_recycling=None):
+    def accept(self, fgraph, no_recycling=None, profile=None):
         """
 
         Parameters
         ----------
         fgraph
-            A PerformLinker can have accepted one FunctionGraph instance at a
-            time.
+            A PerformLinker can have accepted one FunctionGraph instance at a time.
         no_recycling
             WRITEME
 
@@ -775,7 +781,8 @@ class PerformLinker(LocalLinker):
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
-            return type(self)(allow_gc=self.allow_gc).accept(fgraph, no_recycling)
+            return type(self)(allow_gc=self.allow_gc).accept(
+                fgraph, no_recycling, profile)
             # raise Exception("Cannot accept from a Linker that is already tied to another FunctionGraph.")
         self.fgraph = fgraph
         self.no_recycling = no_recycling
@@ -783,13 +790,14 @@ class PerformLinker(LocalLinker):
 
     def make_all(self, input_storage=None, output_storage=None, storage_map=None):
         """
+        Returns Function to run all nodes, list of input containers, list of outputs
 
         Parameters
         ----------
         input_storage
-            WRITEME
+            list of storages corresponding to fgraph.inputs
         output_storage
-            WRITEME
+            list of storages corresponding to fgraph.outputs
 
         Returns
         -------
@@ -815,17 +823,13 @@ class PerformLinker(LocalLinker):
             # the python version
             # Note : ops that implement their own make thunk don't usually
             # have this attribute defiend !!
-            old_value = getattr(node.op, '_op_use_c_code', False)
-            try:
-                node.op._op_use_c_code = False
-                thunks += [node.op.make_thunk(node,
-                                              storage_map,
-                                              compute_map,
-                                              no_recycling)]
-                thunks[-1].inputs = [storage_map[v] for v in node.inputs]
-                thunks[-1].outputs = [storage_map[v] for v in node.outputs]
-            finally:
-                node.op._op_use_c_code = old_value
+            thunks += [node.op.make_thunk(node,
+                                          storage_map,
+                                          compute_map,
+                                          no_recycling,
+                                          'py')]
+            thunks[-1].inputs = [storage_map[v] for v in node.inputs]
+            thunks[-1].outputs = [storage_map[v] for v in node.outputs]
 
         computed, last_user = gc_helper(order)
         if self.allow_gc:
@@ -876,8 +880,6 @@ def add_clear_storage(f, computed, storage_map):
 
 class WrapLinker(Linker):
     """
-    WRITEME
-
     This class makes it easier to run several L{LocalLinker}s in parallel, and
     offers some control over how each thunk is run.
 
@@ -939,7 +941,7 @@ class WrapLinker(Linker):
             linkers=[l.clone(allow_gc=allow_gc) for l in self.linkers],
             wrapper=self.wrapper)
 
-    def accept(self, fgraph, no_recycling=None):
+    def accept(self, fgraph, no_recycling=None, profile=None):
         """
 
         Parameters
