@@ -1,12 +1,16 @@
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 from functools import partial
+from collections import OrderedDict
+
 import sys
 import time
 import inspect
 
+import numpy as np
+from six.moves import StringIO
+
 import theano
 from theano import config
-from theano.compat import OrderedDict
 from theano.gof import graph
 
 
@@ -32,6 +36,181 @@ class ReplacementDidntRemovedError(Exception):
     pass
 
 
+class BadOptimization(Exception):
+    """
+    Exception: some variable and its substitute take different runtime values.
+
+    Note: If there is only 1 parameter and it is a string, we will use
+    it as the error message. This is needed when we catch, extend and
+    reraise an error.
+
+    """
+
+    new_r = None
+    """
+    A `Variable` instance that took a different value from `old_r`,
+    but which replaced `old_r`.
+
+    """
+
+    old_r = None
+    """
+    A `Variable` instance that was replaced by `new_r`.
+
+    """
+
+    old_r_val = None
+    """
+    The value computed for `old_r`.
+
+    """
+
+    new_r_val = None
+    """
+    The value computed for `new_r`.
+
+    """
+
+    reason = None
+    """
+    An object that indicates why old_r was turned into new_r.
+
+    Convention is that this is the name of the optimization that
+    requested the replacement.
+
+    """
+
+    old_graph = ""
+    """
+    A multiline string representation of the graph leading to
+    old_r, at the time of the replacement.
+
+    """
+
+    new_graph = ""
+    """
+    A multiline string representation of the graph leading to
+    new_r, at the time of the replacement.
+
+    """
+
+    def __init__(self, old_r, new_r=None, old_r_val=None, new_r_val=None, reason=None,
+                 old_graph=None, new_graph=None):
+        super(BadOptimization, self).__init__()
+        self.old_r = old_r
+        self.new_r = new_r
+        self.old_r_val = old_r_val
+        self.new_r_val = new_r_val
+        self.reason = reason
+        self.old_graph = old_graph
+        self.new_graph = new_graph
+
+        # To allow extending the error message of an existing error.
+        self.full_err = None
+        if isinstance(old_r, str):
+            assert (new_r is None and old_r_val is None and new_r_val is None and
+                    reason is None and old_graph is None and new_graph is None)
+            self.full_err = old_r
+
+    def __str__(self):
+        return self.str_diagnostic()
+
+    def str_diagnostic(self):
+        """
+        Return a pretty multiline string representating the cause
+        of the exception.
+
+        """
+        # We have a pre-made message
+        if getattr(self, 'full_err', None) is not None:
+            return self.full_err
+        sio = StringIO()
+        val_str_len_limit = 800
+        print("BadOptimization Error", super(BadOptimization,
+                                             self).__str__(), file=sio)
+        print("  Variable: id", id(self.new_r), self.new_r, file=sio)
+        print("  Op", self.new_r.owner, file=sio)
+        print("  Value Type:", type(self.new_r_val), file=sio)
+        try:
+            ssio = StringIO()
+            print("  Old Value shape, dtype, strides:", end=' ', file=ssio)
+            print(self.old_r_val.shape, end=' ', file=ssio)
+            print(self.old_r_val.dtype, end=' ', file=ssio)
+            print(self.old_r_val.strides, file=ssio)
+            # only if all succeeds to we add anything to sio
+            print(ssio.getvalue(), file=sio)
+        except Exception:
+            pass
+
+        str_old_r_val = str(self.old_r_val)
+        if len(str_old_r_val) > val_str_len_limit:
+            print("  Old Value: ", str(self.old_r_val)[
+                :val_str_len_limit], '...', file=sio)
+        else:
+            print("  Old Value: ", str(self.old_r_val), file=sio)
+
+        try:
+            ssio = StringIO()
+            print("  New Value shape, dtype, strides:", end=' ', file=ssio)
+            print(self.new_r_val.shape, end=' ', file=ssio)
+            print(self.new_r_val.dtype, end=' ', file=ssio)
+            print(self.new_r_val.strides, file=ssio)
+            # only if all succeeds to we add anything to sio
+            print(ssio.getvalue(), file=sio)
+        except Exception:
+            pass
+        str_new_r_val = str(self.new_r_val)
+        if len(str_new_r_val) > val_str_len_limit:
+            print("  New Value: ", str(self.new_r_val)[
+                :val_str_len_limit], '...', file=sio)
+        else:
+            print("  New Value: ", str(self.new_r_val), file=sio)
+
+        try:
+            ov = np.asarray(self.old_r_val)
+            nv = np.asarray(self.new_r_val)
+            ssio = StringIO()
+            abs_diff = np.absolute(nv - ov)
+            print("  Max Abs Diff: ", np.max(abs_diff), file=ssio)
+            print("  Mean Abs Diff: ", np.mean(abs_diff), file=ssio)
+            print("  Median Abs Diff: ", np.median(abs_diff), file=ssio)
+            print("  Std Abs Diff: ", np.std(abs_diff), file=ssio)
+            arg_max_val = np.argmax(abs_diff)
+            values_at_max = (nv.flatten()[arg_max_val],
+                             ov.flatten()[arg_max_val])
+            print("  Value at Max Diff: ", values_at_max, file=ssio)
+
+            # N.B. the maximum(..., 1e-8) protects against div by 0 when
+            #      nv == ov == 0
+            reldiff = (abs_diff /
+                       np.maximum(np.absolute(nv) + np.absolute(ov),
+                                  1e-8))
+            print("  Max Rel Diff: ", np.max(reldiff), file=ssio)
+            print("  Mean Rel Diff: ", np.mean(reldiff), file=ssio)
+            print("  Median Rel Diff: ", np.median(reldiff), file=ssio)
+            print("  Std Rel Diff: ", np.std(reldiff), file=ssio)
+            arg_max_val = np.argmax(reldiff)
+            values_at_max = (nv.flatten()[arg_max_val],
+                             ov.flatten()[arg_max_val])
+            print("  Value at Max Diff: ", values_at_max, file=ssio)
+            # only if all succeeds to we add anything to sio
+            print(ssio.getvalue(), file=sio)
+        except Exception:
+            pass
+
+        print("  Reason: ", str(self.reason), file=sio)
+        print("  Old Graph:", file=sio)
+        print(self.old_graph, file=sio)
+        print("  New Graph:", file=sio)
+        print(self.new_graph, file=sio)
+        print("", file=sio)
+        print("Hint: relax the tolerance by setting tensor.cmp_sloppy=1",
+              file=sio)
+        print("  or even tensor.cmp_sloppy=2 for less-strict comparison",
+              file=sio)
+        return sio.getvalue()
+
+
 class Feature(object):
     """
     Base class for FunctionGraph extensions.
@@ -114,10 +293,20 @@ class Feature(object):
 class Bookkeeper(Feature):
 
     def on_attach(self, fgraph):
+        """
+        Called by FunctionGraph.attach_feature, the method that attaches
+        the feature to the FunctionGraph. Since this is called after the
+        FunctionGraph is initially populated, this is where you should
+        run checks on the initial contents of the FunctionGraph.
+        """
         for node in graph.io_toposort(fgraph.inputs, fgraph.outputs):
             self.on_import(fgraph, node, "on_attach")
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         for node in graph.io_toposort(fgraph.inputs, fgraph.outputs):
             self.on_prune(fgraph, node, 'Bookkeeper.detach')
 
@@ -178,6 +367,10 @@ class History(Feature):
         fgraph.revert = partial(self.revert, fgraph)
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         del fgraph.checkpoint
         del fgraph.revert
         del self.history[fgraph]
@@ -223,10 +416,19 @@ class Validator(Feature):
         fgraph.consistent = partial(self.consistent_, fgraph)
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         del fgraph.validate
         del fgraph.consistent
 
     def validate_(self, fgraph):
+        """
+        If the caller is replace_all_validate, just raise the
+        exception. replace_all_validate will print out the
+        verbose output. Or it has to be done here before raise.
+        """
         t0 = time.time()
         try:
             ret = fgraph.execute_callbacks('validate')
@@ -289,6 +491,10 @@ class ReplaceValidate(History, Validator):
             self.replace_all_validate_remove, fgraph)
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         History.on_detach(self, fgraph)
         Validator.on_detach(self, fgraph)
         del self._nodes_removed
@@ -304,6 +510,9 @@ class ReplaceValidate(History, Validator):
         chk = fgraph.checkpoint()
         if verbose is None:
             verbose = config.optimizer_verbose
+        if config.scan.debug:
+            scans = [n for n in fgraph.apply_nodes if isinstance(n.op, theano.scan_module.scan_op.Scan)]
+
         for r, new_r in replacements:
             try:
                 fgraph.replace(r, new_r, reason=reason, verbose=False)
@@ -337,6 +546,14 @@ class ReplaceValidate(History, Validator):
             if verbose:
                 print("validate failed on node %s.\n Reason: %s, %s" % (r, reason, e))
             raise
+        if config.scan.debug:
+            scans2 = [n for n in fgraph.apply_nodes if isinstance(n.op, theano.scan_module.scan_op.Scan)]
+            nb = len(scans)
+            nb2 = len(scans2)
+            if nb2 > nb:
+                print("Extra scan introduced", nb, nb2, getattr(reason, 'name', reason), r, new_r)
+            elif nb2 < nb:
+                print("Scan removed", nb, nb2, getattr(reason, 'name', reason), r, new_r)
         if verbose:
             print(reason, r, new_r)
         # The return is needed by replace_all_validate_remove
@@ -401,6 +618,10 @@ class NodeFinder(Bookkeeper):
         Bookkeeper.on_attach(self, fgraph)
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         if self.fgraph is not fgraph:
             raise Exception("This NodeFinder instance was not attached to the"
                             " provided fgraph.")
@@ -450,6 +671,10 @@ class PrintListener(Feature):
             print("-- attaching to: ", fgraph)
 
     def on_detach(self, fgraph):
+        """
+        Should remove any dynamically added functionality
+        that it installed into the function_graph
+        """
         if self.active:
             print("-- detaching from: ", fgraph)
 
