@@ -5,13 +5,15 @@ A VM is not actually different from a Linker, we just decided
 VM was a better name at some point.
 
 """
+from __future__ import absolute_import, print_function, division
+
 from . import link
 from collections import defaultdict
 import logging
-import os
 import sys
 import time
 import warnings
+import platform
 
 from theano.configparser import (config, _config_var_list)
 
@@ -25,6 +27,9 @@ logger = logging.getLogger(__name__)
 
 def calculate_reallocate_info(order, fgraph, storage_map, compute_map_re,
                               dependencies):
+    """
+    WRITEME : explain the parameters
+    """
     reallocated_info = {}
     viewed_by = {}
     for var in fgraph.variables:
@@ -187,7 +192,9 @@ class VM(object):
         raise NotImplementedError('override me')
 
     def update_profile(self, profile):
-        # accumulate into the profile object
+        """
+        Accumulate into the profile object
+        """
         for node, thunk, t, c in zip(self.nodes, self.thunks,
                                      self.call_times, self.call_counts):
             profile.apply_time.setdefault(node, 0.0)
@@ -201,6 +208,7 @@ class VM(object):
         if hasattr(self, 'variable_shape'):
             profile.variable_shape = self.variable_shape.copy()
             profile.variable_strides = self.variable_strides.copy()
+            profile.variable_offset = self.variable_offset.copy()
 
         if hasattr(self, 'node_executed_order'):
             profile.node_executed_order = self.node_executed_order[:]
@@ -238,7 +246,7 @@ class Loop(VM):
                     t1 = time.time()
                     self.call_counts[i] += 1
                     self.call_times[i] += t1 - t0
-            except:
+            except Exception:
                 link.raise_with_op(node, thunk)
         else:
             for cont in self.pre_call_clear:
@@ -246,7 +254,7 @@ class Loop(VM):
             try:
                 for thunk, node in zip(self.thunks, self.nodes):
                     thunk()
-            except:
+            except Exception:
                 link.raise_with_op(node, thunk)
 
 
@@ -282,7 +290,7 @@ class LoopGC(VM):
                     for old_s in old_storage:
                         old_s[0] = None
                     i += 1
-            except:
+            except Exception:
                 link.raise_with_op(node, thunk)
         else:
             for cont in self.pre_call_clear:
@@ -293,7 +301,7 @@ class LoopGC(VM):
                     thunk()
                     for old_s in old_storage:
                         old_s[0] = None
-            except:
+            except Exception:
                 link.raise_with_op(node, thunk)
 
 
@@ -325,7 +333,8 @@ class Stack(VM):
 
     def __init__(self, nodes, thunks, pre_call_clear,
                  storage_map, compute_map, fgraph, allow_gc,
-                 dependencies=None, callback=None):
+                 n_updates, dependencies=None, callback=None,
+                 callback_input=None):
         super(Stack, self).__init__(nodes, thunks, pre_call_clear)
 
         self.allow_gc = allow_gc
@@ -335,9 +344,12 @@ class Stack(VM):
         self.storage_map = storage_map
         self.variable_shape = {}  # Variable -> shape
         self.variable_strides = {}  # Variable -> strides
+        self.variable_offset = {}  # Variable -> offset
         self.compute_map = compute_map
         self.node_idx = node_idx = {}
         self.callback = callback
+        self.callback_input = callback_input
+        self.n_updates = n_updates
 
         ords = fgraph.orderings()
 
@@ -394,7 +406,7 @@ class Stack(VM):
             )
         return rval, dt
 
-    def __call__(self):
+    def __call__(self, output_subset=None):
         storage_map = self.storage_map
         compute_map = self.compute_map
         thunks = self.thunks
@@ -402,29 +414,45 @@ class Stack(VM):
         self.node_executed_order = []
         self.node_cleared_order = []
 
+        for cont in self.pre_call_clear:
+            cont[0] = None
+
         for k in self.storage_map:
             compute_map[k][0] = (k.owner is None)
+            if self.callback_input and compute_map[k][0]:
+                self.callback_input(k, self.storage_map[k][0])
 
         # apply_stack contains nodes
-        apply_stack = list(self.base_apply_stack)
+        if output_subset is not None:
+            first_updated = len(self.outputs) - self.n_updates
+            output_subset = output_subset + list(range(first_updated,
+                                                       len(self.outputs)))
+            apply_stack =\
+                [self.outputs[i].owner for i in output_subset
+                    if self.outputs[i].owner]
+        else:
+            apply_stack = list(self.base_apply_stack)
+
         last_apply_stack_len = -1
 
-        # This record all function inputs/shared varibles and constants
+        # This record all function inputs/shared variables and constants
         for var, data in iteritems(self.storage_map):
             if data[0] is None:
                 continue
             if hasattr(var.type, 'get_shape_info'):
                 sh = var.type.get_shape_info(data[0])
             else:
-                sh = 'input no shape'
+                sh = 'no shape'
             self.variable_shape[var] = sh
-            st = getattr(data[0], 'strides', 'input no strides')
+            st = getattr(data[0], 'strides', 'no strides')
             if getattr(data[0], 'flags', False) and data[0].flags.c_contiguous:
                 st = 'c'
             elif (hasattr(data[0], 'is_c_contiguous') and
                   data[0].is_c_contiguous()):
                 st = "c"
             self.variable_strides[var] = st
+            off = getattr(data[0], 'offset', '')
+            self.variable_offset[var] = off
 
         while apply_stack:
             # Make sure something happened last time round.  This is
@@ -461,7 +489,7 @@ class Stack(VM):
                     try:
                         _, dt = self.run_thunk_of_node(current_apply)
                         del _
-                        if config.profile:
+                        if config.profile or config.print_global_stats:
                             current_idx = self.node_idx[current_apply]
                             self.call_counts[current_idx] += 1
                             self.call_times[current_idx] += dt
@@ -475,17 +503,19 @@ class Stack(VM):
                                 if hasattr(var.type, 'get_shape_info'):
                                     sh = var.type.get_shape_info(o[0])
                                 else:
-                                    sh = 'input no shape'
+                                    sh = 'no shape'
                                 self.variable_shape[var] = sh
                                 st = getattr(o[0], 'strides',
-                                             'input no strides')
+                                             'no strides')
                                 if (getattr(o[0], 'flags', False) and
                                         o[0].flags.c_contiguous):
                                     st = 'c'
-                                elif (hasattr(data[0], 'is_c_contiguous') and
-                                      data[0].is_c_contiguous()):
+                                elif (hasattr(o[0], 'is_c_contiguous') and
+                                      o[0].is_c_contiguous()):
                                     st = "c"
                                 self.variable_strides[var] = st
+                                off = getattr(o[0], 'offset', '')
+                                self.variable_offset[var] = off
                     except Exception:
                         link.raise_with_op(
                             current_apply,
@@ -575,7 +605,7 @@ class Stack(VM):
                         if current_apply.inputs[r].owner:
                             apply_stack.append(current_apply.inputs[r].owner)
                 else:
-                    if config.profile:
+                    if config.profile or config.print_global_stats:
                         for (idx, o) in enumerate(thunks[
                                 self.node_idx[current_apply]].outputs):
                             var = self.nodes[
@@ -584,16 +614,18 @@ class Stack(VM):
                             if hasattr(var.type, 'get_shape_info'):
                                 sh = var.type.get_shape_info(o[0])
                             else:
-                                sh = 'input no shape'
+                                sh = 'no shape'
                             self.variable_shape[var] = sh
-                            st = getattr(o[0], 'strides', 'input no strides')
+                            st = getattr(o[0], 'strides', 'no strides')
                             if (getattr(o[0], 'flags', False) and
                                     o[0].flags.c_contiguous):
                                 st = 'c'
-                            elif (hasattr(data[0], 'is_c_contiguous') and
-                                  data[0].is_c_contiguous()):
+                            elif (hasattr(o[0], 'is_c_contiguous') and
+                                  o[0].is_c_contiguous()):
                                 st = "c"
                             self.variable_strides[var] = st
+                            off = getattr(o[0], 'offset', '')
+                            self.variable_offset[var] = off
 
                     input_index = []
 
@@ -635,6 +667,10 @@ class Stack(VM):
 
 
 try:
+    # If cxx is explicitely set to an empty string, we do not want to import neither lazylinker C code
+    # nor lazylinker compiled C code from cache.
+    if not theano.config.cxx:
+        raise theano.gof.cmodule.MissingGXX('lazylinker will not be imported if theano.config.cxx is not set.')
     from . import lazylinker_c
 
     class CVM(lazylinker_c.CLazyLinker, VM):
@@ -671,20 +707,29 @@ class VM_Linker(link.LocalLinker):
         A callable object to call after each call to a thunk within
         the virtual machine.  It will be called with four arguments called
         'node', 'thunk', 'storage_map', and 'compute_map'.
+    callback_input
+        A callable object to call on each input to the graph
+        (variables with no owner).  This includes constants and shared
+        variables values.  It will be called with two arguments:
+        'var', 'value'.
     lazy
         Useful only when use_cloop is False. When lazy is None, use the
         theano flag vm.lazy value. Then if we have a None (default) we auto
-        detect if lazy evaluation is needed and use the apropriate
+        detect if lazy evaluation is needed and use the appropriate
         version. If lazy is True or False, we force the version used
         between Loop/LoopGC and Stack.
     c_thunks
         If None or True, don't change the default. If False,
         don't compile c code for the thunks.
+    allow_partial_eval
+        If True, enforces usage of Stack or CVM, to allow for partial
+        evaluation of functions (calculating a subset of outputs).
 
     """
 
     def __init__(self, allow_gc=None, use_cloop=False, callback=None,
-                 lazy=None, schedule=None, c_thunks=None):
+                 callback_input=None, lazy=None, schedule=None,
+                 c_thunks=None, allow_partial_eval=None):
         # Note: if more parameters are added to __init__, make sure to forward
         # them in the "type(self)(...)" call in the "accept" method below.
         if allow_gc is None:
@@ -693,22 +738,53 @@ class VM_Linker(link.LocalLinker):
         self.allow_gc = allow_gc
         self.use_cloop = use_cloop
         self.callback = callback
+        self.callback_input = callback_input
         self.lazy = lazy
+        if c_thunks is None:
+            c_thunks = bool(theano.config.cxx)
         self.c_thunks = c_thunks
+        self.allow_partial_eval = allow_partial_eval
         self.updated_vars = {}
         if schedule:
             self.schedule = schedule
 
-    def accept(self, fgraph, no_recycling=None):
-        """
+    def accept(self, fgraph, no_recycling=None, profile=None):
+        """Check if fgraph is the first FunctionGraph that has ever been
+        associated to self, else, create a new VM_Linker
+        associated to fgraph
 
         Parameters
         ----------
         fgraph
             A PerformLinker can have accepted one FunctionGraph instance
             at a time.
+
         no_recycling
-            WRITEME
+
+            no_recycling is a list of storage (list of 1 element, the
+            value corresponding to one variable). Those variable
+            storage should not be reused after the call that created
+            them.
+
+            This happen for example for output of the graph that we
+            give to the user. We don't want to reuse those object in
+            case the user have kept it.
+
+            VM_Linker make sure this happen by setting the list
+            element to None at the start of each call.
+
+            Older Linker use not exactly the same mechanism. They will
+            also modify the c code to don't look up the value in the
+            storage. This cause duplicate c code compilation for the
+            same op if they are in the middle of the graph or in the
+            no_recycling. We don't want that, so compile all c code
+            the same (middle of the graph vs output).
+
+            TODO: change the logic to remove the reference at the end
+            of the call instead of the start. This will request all VM
+            implementation (Loop, LoopGC, Stack, CVM).__call__ to
+            return the user outputs as Function.__call__ won't be able
+            to find them anymore.
 
         Returns
         -------
@@ -716,19 +792,6 @@ class VM_Linker(link.LocalLinker):
         associated to self, else, a new VM_Linker associated to fgraph.
 
         """
-        if (config.profile and
-                hasattr(theano, 'sandbox') and
-                hasattr(theano.sandbox, 'cuda') and
-                theano.sandbox.cuda.cuda_enabled):
-            if os.environ.get('CUDA_LAUNCH_BLOCKING', '0') != '1':
-                raise Exception(
-                    "You are running the Theano profiler with CUDA enabled."
-                    " Theano GPU ops execution is asynchronous by default."
-                    " So by default, the profile is useless."
-                    " You must set the environment variable"
-                    " CUDA_LAUNCH_BLOCKING to 1 to tell the CUDA driver to"
-                    " synchronize the execution to get a meaningful profile.")
-
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
@@ -739,12 +802,16 @@ class VM_Linker(link.LocalLinker):
                 allow_gc=self.allow_gc,
                 use_cloop=self.use_cloop,
                 callback=self.callback,
+                callback_input=self.callback_input,
                 lazy=self.lazy,
                 schedule=self.schedule,
                 c_thunks=self.c_thunks,
-            ).accept(fgraph, no_recycling)
+                allow_partial_eval=self.allow_partial_eval
+            ).accept(fgraph, no_recycling, profile)
         self.fgraph = fgraph
         self.no_recycling = no_recycling
+        self.profile = profile
+
         return self
 
     def accept_var_updates(self, updated_vars):
@@ -808,22 +875,30 @@ class VM_Linker(link.LocalLinker):
 
         pre_call_clear = [storage_map[v] for v in self.no_recycling]
 
-        if (self.callback is not None or
-                (config.profile and config.profile_memory)):
+        if (self.callback is not None or self.callback_input is not None or
+                ((config.profile or config.print_global_stats) and config.profile_memory) or
+                (self.allow_partial_eval and not self.use_cloop)):
 
-            if self.use_cloop and self.callback is not None:
+            if self.use_cloop and (self.callback is not None or
+                                   self.callback_input is not None):
                 logger.warn('CVM does not support callback, using Stack VM.')
             if self.use_cloop and config.profile_memory:
                 warnings.warn(
                     'CVM does not support memory profile, using Stack VM.')
+            if not self.use_cloop and self.allow_partial_eval:
+                warnings.warn(
+                    'LoopGC does not support partial evaluation, '
+                    'using Stack VM.')
             # Needed for allow_gc=True, profiling and storage_map reuse
             deps = self.compute_gc_dependencies(storage_map)
             vm = Stack(
                 nodes, thunks, pre_call_clear,
                 storage_map, compute_map,
                 self.fgraph, self.allow_gc,
+                len(updated_vars),
                 dependencies=deps,
-                callback=self.callback)
+                callback=self.callback,
+                callback_input=self.callback_input)
         elif self.use_cloop:
             # create a map from nodes to ints and vars to ints
             nodes_idx = {}
@@ -909,7 +984,11 @@ class VM_Linker(link.LocalLinker):
                 if oidx in update_in_from_out:
                     update_storage.append(update_in_from_out[oidx])
 
-            c0 = sys.getrefcount(node_n_inputs)
+            # PyPy has no sys.getrefcount, so ignore this check if not running
+            # under CPython.
+            if platform.python_implementation() == 'CPython':
+                c0 = sys.getrefcount(node_n_inputs)
+
             vm = CVM(
                 nodes,
                 thunks,
@@ -932,7 +1011,9 @@ class VM_Linker(link.LocalLinker):
                 update_storage=update_storage,
                 dependencies=dependency_map_list,
             )
-            assert c0 == sys.getrefcount(node_n_inputs)
+
+            if platform.python_implementation() == 'CPython':
+                assert c0 == sys.getrefcount(node_n_inputs)
         else:
             lazy = self.lazy
             if lazy is None:
@@ -961,7 +1042,8 @@ class VM_Linker(link.LocalLinker):
                     nodes, thunks, pre_call_clear,
                     storage_map, compute_map,
                     self.fgraph, self.allow_gc,
-                    dependencies=deps
+                    len(updated_vars),
+                    dependencies=deps,
                 )
         return vm
 
@@ -970,7 +1052,6 @@ class VM_Linker(link.LocalLinker):
                  ):
         fgraph = self.fgraph
         order = self.schedule(fgraph)
-        no_recycling = self.no_recycling
 
         input_storage, output_storage, storage_map = link.map_storage(
             fgraph, order, input_storage, output_storage, storage_map)
@@ -992,15 +1073,23 @@ class VM_Linker(link.LocalLinker):
 
         reallocated_info = calculate_reallocate_info(
             order, fgraph, storage_map, compute_map_re, dependencies)
-
+        t0 = time.time()
+        linker_make_thunk_time = {}
+        impl = None
+        if self.c_thunks is False:
+            impl = 'py'
         for node in order:
             try:
-                if self.c_thunks is False:
-                    node.op._op_use_c_code = False
+                thunk_start = time.time()
+                # no-recycling is done at each VM.__call__ So there is
+                # no need to cause duplicate c code by passing
+                # no_recycling here.
                 thunks.append(node.op.make_thunk(node,
                                                  storage_map,
                                                  compute_map,
-                                                 no_recycling))
+                                                 [],
+                                                 impl=impl))
+                linker_make_thunk_time[node] = time.time() - thunk_start
                 if not hasattr(thunks[-1], 'lazy'):
                     # We don't want all ops maker to think about lazy Ops.
                     # So if they didn't specify that its lazy or not, it isn't.
@@ -1010,6 +1099,12 @@ class VM_Linker(link.LocalLinker):
                 e.args = ("The following error happened while"
                           " compiling the node", node, "\n") + e.args
                 raise
+        t1 = time.time()
+
+        if self.profile:
+            self.profile.linker_node_make_thunks += t1 - t0
+            self.profile.linker_make_thunk_time = linker_make_thunk_time
+
         for node, thunk in zip(order, thunks):
             thunk.inputs = [storage_map[v] for v in node.inputs]
             thunk.outputs = [storage_map[v] for v in node.outputs]
@@ -1019,8 +1114,8 @@ class VM_Linker(link.LocalLinker):
             lazy = config.vm.lazy
         if lazy is None:
             lazy = not all([(not th.lazy) for th in thunks])
-        if not (lazy or (config.profile and config.profile_memory) or
-                self.use_cloop or self.callback):
+        if not (lazy or ((config.profile or config.print_global_stats) and config.profile_memory) or
+                self.use_cloop or self.callback or self.callback_input):
             for pair in itervalues(reallocated_info):
                 storage_map[pair[1]] = storage_map[pair[0]]
 
@@ -1048,6 +1143,7 @@ class VM_Linker(link.LocalLinker):
                           )
 
         vm.storage_map = storage_map
+        vm.compute_map = compute_map
 
         return (vm,
                 [link.Container(input, storage)
@@ -1061,3 +1157,7 @@ class VM_Linker(link.LocalLinker):
         self.__dict__.update(d)
         if not hasattr(self, 'c_thunks'):
             self.c_thunks = True
+        if not hasattr(self, 'allow_partial_eval'):
+            self.allow_partial_eval = None
+        if not hasattr(self, 'callback_input'):
+            self.callback_input = None
