@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
+
 __docformat__ = 'restructuredtext en'
 
 import difflib
