@@ -1,7 +1,8 @@
+from __future__ import absolute_import, print_function, division
 import unittest
 
 import theano
-import numpy
+import numpy as np
 import scipy.sparse as sp
 
 from theano import sparse
@@ -15,7 +16,6 @@ from theano.sparse import SparseType, dense_from_sparse, transpose
 
 from theano.sparse.tests.test_basic import sparse_random_inputs
 from theano.tests import unittest_tools as utt
-from theano.sparse import verify_grad_sparse
 
 # To maintain compatibility
 from theano.sparse.basic import TrueDot, true_dot
