@@ -1,3 +1,4 @@
+from __future__ import absolute_import, print_function, division
 from theano.gof.utils import hash_from_code
 
 
@@ -8,7 +9,7 @@ def hash_from_sparse(data):
     # We also need to add the dtype to make the distinction between
     # uint32 and int32 of zeros with the same shape.
 
-    # Python hash is not strong, so I always use md5. To avoid having a too
+    # Python hash is not strong, so use sha256 instead. To avoid having a too
     # long hash, I call it again on the contatenation of all parts.
     return hash_from_code(hash_from_code(data.data) +
                           hash_from_code(data.indices) +
