@@ -1,13 +1,14 @@
-from __future__ import print_function
+from __future__ import absolute_import, print_function, division
 import unittest
 
-import numpy
+import numpy as np
 
 from theano.compat import imap
 import theano.tensor.inplace
 from theano.tensor import basic as tensor
 from theano import tensor as T
 from theano import config
+from theano.gof.opt import check_stack_trace
 from theano.tests import unittest_tools as utt
 from theano.tensor.nnet import (sigmoid, sigmoid_inplace,
                                 softplus, ultra_fast_sigmoid, hard_sigmoid)
@@ -15,7 +16,7 @@ from theano.tensor.nnet.sigm import (
     compute_mul, is_1pexp, parse_mul_tree, perform_sigm_times_exp,
     register_local_1msigmoid, simplify_mul,
 )
-from theano.tensor.tests.test_basic import (makeBroadcastTester, rand,
+from theano.tensor.tests.test_basic import (makeBroadcastTester, copymod,
                                             check_floatX, upcast_int8_nfunc,
                                             _good_broadcast_unary_normal_no_complex)
 
@@ -25,14 +26,16 @@ class T_sigmoid(unittest.TestCase):
         utt.seed_rng()
 
     def test_elemwise(self):
-        utt.verify_grad(sigmoid, [numpy.random.rand(3, 4)])
-
+        utt.verify_grad(sigmoid, [np.random.rand(3, 4)])
 
 SigmoidTester = makeBroadcastTester(
     op=sigmoid,
     expected=upcast_int8_nfunc(lambda inputs: check_floatX(
-        inputs, 1 / (1 + numpy.exp(-inputs)))),
-    good=_good_broadcast_unary_normal_no_complex,
+        inputs, 1 / (1 + np.exp(-inputs)))),
+    good=copymod(_good_broadcast_unary_normal_no_complex,
+                 without=['uint16']),  # The reason that 'uint16' is excluted is that
+                                       # theano works well but numpy overflows resulting
+                                       # in an assertion error.
     # grad=_grad_broadcast_unary_normal,
     name='SigmoidTester',
 )
@@ -40,30 +43,34 @@ SigmoidTester = makeBroadcastTester(
 UltraFastSigmoidTester = makeBroadcastTester(
     op=ultra_fast_sigmoid,
     expected=upcast_int8_nfunc(lambda inputs: check_floatX(
-        inputs, 1 / (1 + numpy.exp(-inputs)))),
-    good=_good_broadcast_unary_normal_no_complex,
+        inputs, 1 / (1 + np.exp(-inputs)))),
+    good=copymod(_good_broadcast_unary_normal_no_complex,
+                 without=['uint16']),  # numpy fucnting overflows with uint16.
     # grad=_grad_broadcast_unary_normal,
     name='UltraFastSigmoidTester',
-# This is an approx of the sigmoid. That is why we raise eps
+    # This is an approx of the sigmoid. That is why we raise eps
     eps=5e-2)
 
 HardSigmoidTester = makeBroadcastTester(
     op=hard_sigmoid,
     expected=upcast_int8_nfunc(lambda inputs: check_floatX(
-        inputs, 1 / (1 + numpy.exp(-inputs)))),
-    good=_good_broadcast_unary_normal_no_complex,
+        inputs, 1 / (1 + np.exp(-inputs)))),
+    good=copymod(_good_broadcast_unary_normal_no_complex,
+                 without=['uint16']),  # numpy fucnting overflows with uint16.
     # grad=_grad_broadcast_unary_normal,
     name='HardSigmoidTester',
-# This is an approx of the sigmoid. That is why we raise eps
+    # This is an approx of the sigmoid. That is why we raise eps
     eps=1e-1)
 
 
 SoftplusTester = makeBroadcastTester(
     op=softplus,
     expected=upcast_int8_nfunc(lambda inputs: check_floatX(
-        inputs, numpy.log1p(numpy.exp(inputs)))),
-    good=dict(_good_broadcast_unary_normal_no_complex,
-              int8=[numpy.arange(-127, 89, dtype='int8')]),
+        inputs, np.log1p(np.exp(inputs)))),
+    good=dict(copymod(_good_broadcast_unary_normal_no_complex,
+                      without=['uint8', 'uint16']),  # numpy fucnting overflows with uint16.
+              uint8=[np.arange(0, 89, dtype='uint8')],  # the range is different in new added uint8.
+              int8=[np.arange(-127, 89, dtype='int8')]),
     # grad=_grad_broadcast_unary_normal,
     name='SoftplusTester',
 )
@@ -74,7 +81,7 @@ class T_softplus(unittest.TestCase):
         utt.seed_rng()
 
     def test_elemwise(self):
-        utt.verify_grad(softplus, [numpy.random.rand(3, 4)])
+        utt.verify_grad(softplus, [np.random.rand(3, 4)])
 
 
 class T_sigmoid_opts(unittest.TestCase):
@@ -105,7 +112,7 @@ class T_sigmoid_opts(unittest.TestCase):
         m = self.get_mode(excluding=['local_elemwise_fusion'])
 
         x = T.vector()
-        data = numpy.random.rand(54).astype(config.floatX)
+        data = np.random.rand(54).astype(config.floatX)
 
         backup = config.warn.identify_1pexp_bug
         config.warn.identify_1pexp_bug = False
@@ -126,42 +133,39 @@ class T_sigmoid_opts(unittest.TestCase):
 
             # tests inv_1_plus_exp
             f = theano.function([x], T.fill(x, 1.0) / (1 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+            # todo: solve issue #4589 first
+            # assert check_stack_trace(f, ops_to_check=sigmoid)
             assert [node.op for node in f.maker.fgraph.toposort()] == [sigmoid]
             f(data)
             f = theano.function([x], T.fill(x, 1.0) / (2 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
             assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid]
             f(data)
             f = theano.function([x], T.fill(x, 1.0) / (1 - T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
             assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid]
             f(data)
             f = theano.function([x], T.fill(x, 1.1) / (1 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
             assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid]
             f(data)
 
             # tests inv_1_plus_exp with neg
             f = theano.function([x], T.fill(x, -1.0) / (1 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] == [sigmoid,
-                    theano.tensor.inplace.neg_inplace]
+            # todo: solve issue #4589 first
+            # assert check_stack_trace(
+            #     f, ops_to_check=[sigmoid, theano.tensor.inplace.neg_inplace])
+            assert ([node.op for node in f.maker.fgraph.toposort()] ==
+                    [sigmoid, theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], T.fill(x, -1.0) / (1 - T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], T.fill(x, -1.0) / (2 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], T.fill(x, -1.1) / (1 + T.exp(-x)), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    theano.tensor.inplace.neg_inplace])
             f(data)
 
             # tests double inv_1_plus_exp with neg
@@ -170,39 +174,35 @@ class T_sigmoid_opts(unittest.TestCase):
             # = - (sigm(x) * sigm(x))
             f = theano.function([x], (T.fill(x, -1.0) * T.exp(x)) /
                                 ((1 + T.exp(x)) * (1 + T.exp(-x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] == [sigmoid,
-                    T.mul]
+            # todo: solve issue #4589 first
+            # assert check_stack_trace(f, ops_to_check=[sigmoid, T.mul])
+            assert ([node.op for node in f.maker.fgraph.toposort()] == [sigmoid,
+                    T.mul])
             f(data)
             f = theano.function([x], (T.fill(x, -1.1) * T.exp(x)) /
                                 ((1 + T.exp(x)) * (1 + T.exp(-x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    T.mul, theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    T.mul, theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], (T.fill(x, -1.0) * T.exp(x)) /
                                 ((2 + T.exp(x)) * (1 + T.exp(-x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    T.mul, theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    T.mul, theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], (T.fill(x, -1.0) * T.exp(x)) /
                                 ((1 + T.exp(x)) * (2 + T.exp(-x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    T.mul, theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    T.mul, theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], (T.fill(x, -1.0) * T.exp(x)) /
                                 ((1 + T.exp(x)) * (1 + T.exp(x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    T.mul, theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    T.mul, theano.tensor.inplace.neg_inplace])
             f(data)
             f = theano.function([x], (T.fill(x, -1.0) * T.exp(x)) /
                                 ((1 + T.exp(x)) * (2 + T.exp(-x))), mode=m)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-            assert [node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
-                    T.mul, theano.tensor.inplace.neg_inplace]
+            assert ([node.op for node in f.maker.fgraph.toposort()] != [sigmoid,
+                    T.mul, theano.tensor.inplace.neg_inplace])
             f(data)
 
         finally:
@@ -218,22 +218,21 @@ class T_sigmoid_opts(unittest.TestCase):
 
         # tests exp_over_1_plus_exp
         f = theano.function([x], 1 - T.exp(x) / (1 + T.exp(x)), mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+        assert check_stack_trace(f, ops_to_check=[tensor.neg, sigmoid_inplace])
         assert [node.op for node in f.maker.fgraph.toposort()] == [
             tensor.neg, sigmoid_inplace]
 
         # tests inv_1_plus_exp
         f = theano.function([x], 1 - T.fill(x, 1.0) / (1 + T.exp(-x)), mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-        assert [node.op for node in f.maker.fgraph.toposort()] == [tensor.neg,
-                sigmoid_inplace]
+        assert check_stack_trace(f, ops_to_check=[tensor.neg, sigmoid_inplace])
+        assert ([node.op for node in f.maker.fgraph.toposort()] == [tensor.neg,
+                sigmoid_inplace])
 
     def test_local_sigm_times_exp(self):
-        """
-        Test the `local_sigm_times_exp` optimization.
-        exp(x) * sigm(-x) -> sigm(x)
-        exp(-x) * sigm(x) -> sigm(-x)
-        """
+        # Test the `local_sigm_times_exp` optimization.
+        # exp(x) * sigm(-x) -> sigm(x)
+        # exp(-x) * sigm(x) -> sigm(-x)
+
         def match(func, ops):
             # print [node.op.scalar_op for node in func.maker.fgraph.toposort()]
             assert [node.op for node in func.maker.fgraph.toposort()] == ops
@@ -241,33 +240,34 @@ class T_sigmoid_opts(unittest.TestCase):
         x, y = tensor.vectors('x', 'y')
 
         f = theano.function([x], sigmoid(-x) * tensor.exp(x), mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
         match(f, [sigmoid])
+        assert check_stack_trace(f, ops_to_check=sigmoid)
 
         f = theano.function([x], sigmoid(x) * tensor.exp(-x), mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
         match(f, [tensor.neg, sigmoid])
+        assert check_stack_trace(f, ops_to_check=sigmoid)
 
         f = theano.function([x], -(-(-(sigmoid(x)))) * tensor.exp(-x), mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
         match(f, [tensor.neg, sigmoid, tensor.neg])
+        # assert check_stack_trace(f, ops_to_check=sigmoid)
 
         f = theano.function(
-                [x, y],
-                (sigmoid(x) * sigmoid(-y) * -tensor.exp(-x) *
-                 tensor.exp(x * y) * tensor.exp(y)),
-                mode=m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
-        match(f, [sigmoid, tensor.mul, tensor.neg, tensor.exp, sigmoid,
-                  tensor.mul])
+            [x, y],
+            (sigmoid(x) * sigmoid(-y) * -tensor.exp(-x) *
+                tensor.exp(x * y) * tensor.exp(y)), mode=m)
+        topo = f.maker.fgraph.toposort()
+        for op, nb in [(sigmoid, 2), (tensor.mul, 2),
+                       (tensor.neg, 1), (tensor.exp, 1)]:
+            assert sum([n.op == op for n in topo]) == nb
+        # assert check_stack_trace(f, ops_to_check=[sigmoid, tensor.mul,
+        #                                           tensor.exp])
 
     def test_perform_sigm_times_exp(self):
-        """
-        Test the core function doing the `sigm_times_exp` optimization.
+        # Test the core function doing the `sigm_times_exp` optimization.
+        #
+        # It is easier to test different graph scenarios this way than by
+        # compiling a theano function.
 
-        It is easier to test different graph scenarios this way than by
-        compiling a theano function.
-        """
         x, y, z, t = tensor.vectors('x', 'y', 'z', 't')
         exp = tensor.exp
 
@@ -276,8 +276,8 @@ class T_sigmoid_opts(unittest.TestCase):
             perform_sigm_times_exp(trees[0])
             trees[0] = simplify_mul(trees[0])
             good = theano.gof.graph.is_same_graph(
-                    compute_mul(trees[0]),
-                    compute_mul(trees[1]))
+                compute_mul(trees[0]),
+                compute_mul(trees[1]))
             if not good:
                 print(trees[0])
                 print(trees[1])
@@ -291,11 +291,11 @@ class T_sigmoid_opts(unittest.TestCase):
            -x * sigmoid(-x) * (y * (-1 * z)))
         ok(-sigmoid(-x) *
            (exp(y) * (-exp(-z) * 3 * -exp(x)) *
-            (y * 2 * (-sigmoid(-y) * (z + t) * exp(z)) * sigmoid(z))) *
-           -sigmoid(x),
+            (y * 2 * (-sigmoid(-y) * (z + t) * exp(z)) * sigmoid(z))) * -
+           sigmoid(x),
            sigmoid(x) *
            (-sigmoid(y) * (-sigmoid(-z) * 3) * (y * 2 * ((z + t) * exp(z)))) *
-           -sigmoid(x))
+           (-sigmoid(x)))
         ok(exp(-x) * -exp(-x) * (-sigmoid(x) * -sigmoid(x)),
            -sigmoid(-x) * sigmoid(-x))
         ok(-exp(x) * -sigmoid(-x) * -exp(-x),
@@ -318,9 +318,8 @@ class T_sigmoid_opts(unittest.TestCase):
         mode = self.get_mode()
         if not isinstance(mode, theano.compile.DebugMode):
             f = theano.function([x, lr], ux, mode=mode)
-            assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
             ux_v = f([[50]], 0.1)
-            assert not numpy.isnan(ux_v)
+            assert not np.isnan(ux_v)
 
     def test_local_ultra_fast_sigmoid(self):
         x = tensor.matrix('x')
@@ -328,18 +327,18 @@ class T_sigmoid_opts(unittest.TestCase):
 
         mode = self.get_mode('local_ultra_fast_sigmoid')
         f = theano.function([x], s, mode=mode)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+        assert check_stack_trace(f, ops_to_check=sigmoid)
         topo = f.maker.fgraph.toposort()
         assert len(topo) == 1
         assert topo[0].op == sigmoid
 
         mode = self.get_mode().including('local_ultra_fast_sigmoid')
         f = theano.function([x], s, mode=mode)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+        assert check_stack_trace(f, ops_to_check=ultra_fast_sigmoid)
         topo = f.maker.fgraph.toposort()
         assert topo[0].op == ultra_fast_sigmoid
         assert len(topo) == 1
-        ux_v = f([[-50, -10, -4, -1, 0, 1, 4, 10, 50]])
+        f([[-50, -10, -4, -1, 0, 1, 4, 10, 50]])
 
     def test_local_hard_sigmoid(self):
         x = tensor.matrix('x')
@@ -347,18 +346,20 @@ class T_sigmoid_opts(unittest.TestCase):
 
         mode = self.get_mode('local_hard_sigmoid')
         f = theano.function([x], s, mode=mode)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+        assert check_stack_trace(f, ops_to_check=sigmoid)
         topo = f.maker.fgraph.toposort()
         assert topo[0].op == sigmoid
         assert len(topo) == 1
 
         mode = self.get_mode().including('local_hard_sigmoid')
         f = theano.function([x], s, mode=mode)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
         topo = f.maker.fgraph.toposort()
-        assert len(topo) > 1
         assert not any([n.op == sigmoid for n in topo])
-        ux_v = f([[-50, -10, -4, -1, 0, 1, 4, 10, 50]])
+        f([[-50, -10, -4, -1, 0, 1, 4, 10, 50]])
+
+        mode2 = mode.excluding('fusion').excluding('inplace')
+        f2 = theano.function([x], s, mode=mode2)
+        self.assertTrue(check_stack_trace(f2, ops_to_check=theano.tensor.clip))
 
 
 class T_softplus_opts(unittest.TestCase):
@@ -377,14 +378,18 @@ class T_softplus_opts(unittest.TestCase):
 
         out = T.log(sigmoid(x))
         f = theano.function([x], out, mode=self.m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+
+        # Fix ticket #4581 first
+        # assert check_stack_trace(
+        #     f, ops_to_check=(theano.scalar.Neg,
+        #                      theano.tensor.nnet.sigm.ScalarSoftplus))
         topo = f.maker.fgraph.toposort()
         assert len(topo) == 3
         assert isinstance(topo[0].op.scalar_op, theano.scalar.Neg)
         assert isinstance(topo[1].op.scalar_op,
                           theano.tensor.nnet.sigm.ScalarSoftplus)
         assert isinstance(topo[2].op.scalar_op, theano.scalar.Neg)
-        f(numpy.random.rand(54).astype(config.floatX))
+        f(np.random.rand(54).astype(config.floatX))
 
     def test_log1msigm_to_softplus(self):
         x = T.matrix()
@@ -396,30 +401,32 @@ class T_softplus_opts(unittest.TestCase):
         assert isinstance(topo[0].op.scalar_op,
                           theano.tensor.nnet.sigm.ScalarSoftplus)
         assert isinstance(topo[1].op.scalar_op, theano.scalar.Neg)
-        f(numpy.random.rand(54, 11).astype(config.floatX))
+        # assert check_stack_trace(f, ops_to_check='all')
+        f(np.random.rand(54, 11).astype(config.floatX))
 
         # Same test with a flatten
         out = T.log(1 - T.flatten(sigmoid(x)))
         f = theano.function([x], out, mode=self.m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+
+        # assert check_stack_trace(f, ops_to_check='all')
         topo = f.maker.fgraph.toposort()
         assert len(topo) == 3
         assert tensor.is_flat(topo[0].outputs[0])
         assert isinstance(topo[1].op.scalar_op,
                           theano.tensor.nnet.sigm.ScalarSoftplus)
         assert isinstance(topo[2].op.scalar_op, theano.scalar.Neg)
-        f(numpy.random.rand(54, 11).astype(config.floatX))
+        f(np.random.rand(54, 11).astype(config.floatX))
 
         # Same test with a reshape
         out = T.log(1 - sigmoid(x).reshape([x.size]))
         f = theano.function([x], out, mode=self.m)
         topo = f.maker.fgraph.toposort()
-        #assert len(topo) == 3
+        # assert len(topo) == 3
         assert any(isinstance(node.op, T.Reshape) for node in topo)
         assert any(isinstance(getattr(node.op, 'scalar_op', None),
                               theano.tensor.nnet.sigm.ScalarSoftplus)
                    for node in topo)
-        f(numpy.random.rand(54, 11).astype(config.floatX))
+        f(np.random.rand(54, 11).astype(config.floatX))
 
     def test_log1pexp_to_softplus(self):
         m = theano.config.mode
@@ -430,16 +437,17 @@ class T_softplus_opts(unittest.TestCase):
 
         out = T.log(1 + T.exp(x))
         f = theano.function([x], out, mode=self.m)
-        assert hasattr(f.maker.fgraph.outputs[0].tag, 'trace')
+
+        # Fix ticket #4581 first
+        # assert check_stack_trace(f, ops_to_check='all')
         topo = f.maker.fgraph.toposort()
         assert len(topo) == 1
         assert isinstance(topo[0].op.scalar_op,
                           theano.tensor.nnet.sigm.ScalarSoftplus)
-        f(numpy.random.rand(54).astype(config.floatX))
+        f(np.random.rand(54).astype(config.floatX))
 
 
 class T_sigmoid_utils(unittest.TestCase):
-
     """
     Test utility functions found in 'sigm.py'.
     """
@@ -450,7 +458,7 @@ class T_sigmoid_utils(unittest.TestCase):
         mul_tree = parse_mul_tree(tree)
         assert parse_mul_tree(compute_mul(mul_tree)) == mul_tree
         assert theano.gof.graph.is_same_graph(
-                                    compute_mul(parse_mul_tree(tree)), tree)
+            compute_mul(parse_mul_tree(tree)), tree)
 
     def test_parse_mul_tree(self):
         x, y, z = tensor.vectors('x', 'y', 'z')
@@ -459,7 +467,7 @@ class T_sigmoid_utils(unittest.TestCase):
         assert parse_mul_tree(-x * y) == [False, [[True, x], [False, y]]]
         assert parse_mul_tree(-x) == [True, x]
         assert parse_mul_tree((x * y) * -z) == [
-                        False, [[False, [[False, x], [False, y]]], [True, z]]]
+            False, [[False, [[False, x], [False, y]]], [True, z]]]
 
     def test_is_1pexp(self):
         backup = config.warn.identify_1pexp_bug
@@ -467,15 +475,17 @@ class T_sigmoid_utils(unittest.TestCase):
         try:
             x = tensor.vector('x')
             exp = tensor.exp
-            assert is_1pexp(1 + exp(x)) == (False, x)
-            assert is_1pexp(exp(x) + 1) == (False, x)
-            for neg, exp_arg in imap(is_1pexp, [(1 + exp(-x)), (exp(-x) + 1)]):
+            assert is_1pexp(1 + exp(x), False) == (False, x)
+            assert is_1pexp(exp(x) + 1, False) == (False, x)
+            for neg, exp_arg in imap(lambda x:
+                                     is_1pexp(x, only_process_constants=False),
+                                     [(1 + exp(-x)), (exp(-x) + 1)]):
                 assert not neg and theano.gof.graph.is_same_graph(exp_arg, -x)
-            assert is_1pexp(1 - exp(x)) is None
-            assert is_1pexp(2 + exp(x)) is None
-            assert is_1pexp(exp(x) + 2) is None
-            assert is_1pexp(exp(x) - 1) is None
-            assert is_1pexp(-1 + exp(x)) is None
-            assert is_1pexp(1 + 2 * exp(x)) is None
+            assert is_1pexp(1 - exp(x), False) is None
+            assert is_1pexp(2 + exp(x), False) is None
+            assert is_1pexp(exp(x) + 2, False) is None
+            assert is_1pexp(exp(x) - 1, False) is None
+            assert is_1pexp(-1 + exp(x), False) is None
+            assert is_1pexp(1 + 2 * exp(x), False) is None
         finally:
             config.warn.identify_1pexp_bug = backup
