@@ -1,4 +1,5 @@
-import numpy
+from __future__ import absolute_import, print_function, division
+import numpy as np
 import six.moves.cPickle as pickle
 from six.moves import xrange
 import theano
@@ -12,10 +13,10 @@ def test_no_reuse():
     f = theano.function([x, y], x + y)
 
     # provide both inputs in the first call
-    f(numpy.ones(10, dtype='int64'), numpy.ones(10, dtype='int64'))
+    f(np.ones(10, dtype='int64'), np.ones(10, dtype='int64'))
 
     try:
-        f(numpy.ones(10))
+        f(np.ones(10))
     except TypeError:
         return
     assert not 'should not get here'
@@ -48,13 +49,15 @@ def test_gc_never_pickles_temporaries():
         g = theano.function([x], r, mode=theano.Mode(optimizer=optimizer,
                                                      linker=g_linker))
 
-        len_pre_f = len(pickle.dumps(f))
-        len_pre_g = len(pickle.dumps(g))
+        pre_f = pickle.dumps(f)
+        pre_g = pickle.dumps(g)
+        len_pre_f = len(pre_f)
+        len_pre_g = len(pre_g)
 
         # We can't compare the content or the length of the string
         # between f and g. 2 reason, we store some timming information
         # in float. They won't be the same each time. Different float
-        # can have different lenght when printed.
+        # can have different length when printed.
 
         def a(fn):
             return len(pickle.dumps(fn.maker))
@@ -75,8 +78,8 @@ def test_gc_never_pickles_temporaries():
 
         # now run the function once to create temporaries within the no-gc
         # linker
-        f(numpy.ones(100, dtype='float64'))
-        g(numpy.ones(100, dtype='float64'))
+        f(np.ones(100, dtype='float64'))
+        g(np.ones(100, dtype='float64'))
 
         # serialize the functions again
         post_f = pickle.dumps(f)
@@ -86,7 +89,7 @@ def test_gc_never_pickles_temporaries():
 
         # assert that f() didn't cause the function to grow
         # allow_gc should leave the function un-changed by calling
-        assert len_pre_f == len_post_f
+        assert len_pre_f == len_post_f, (len_pre_f, len_post_f)
 
         # assert that g() didn't cause g to grow because temporaries
         # that weren't collected shouldn't be pickled anyway
@@ -97,13 +100,12 @@ def test_gc_never_pickles_temporaries():
 
 
 def test_merge_opt_runtime():
-    """In the original merge optimization, the following graph took
-    like caused the MERGE optimizer to exhibit really bad performance
-    (quadratic? exponential?)
+    # In the original merge optimization, the following graph took
+    # like caused the MERGE optimizer to exhibit really bad performance
+    # (quadratic? exponential?)
+    #
+    # Ironically, there is actually no merging to do in this graph.
 
-    Ironically, there is actually no merging to do in this graph.
-
-    """
     x = T.dvector()
     for i in xrange(50):
         if i:
