@@ -1,4 +1,5 @@
 """ This file don't test everything. It only test one past crash error."""
+from __future__ import absolute_import, print_function, division
 import theano
 from theano.gof import Constant
 from theano.tensor.type_other import MakeSlice, make_slice, NoneTypeT, NoneConst
@@ -16,10 +17,9 @@ def test_make_slice_merge():
 
 
 def test_none_Constant():
-    """ Tests equals
+    # Tests equals
+    # We had an error in the past with unpickling
 
-    We had an error in the past with unpickling
-    """
     o1 = Constant(NoneTypeT(), None, name='NoneConst')
     o2 = Constant(NoneTypeT(), None, name='NoneConst')
     assert o1.equals(o2)
