@@ -1,5 +1,6 @@
+from __future__ import absolute_import, print_function, division
+from collections import OrderedDict
 from theano.tests.record import RecordMode, Record
-from theano.compat import OrderedDict
 from theano.tests import disturb_mem
 import numpy as np
 import theano
@@ -56,7 +57,7 @@ def test_determinism_1():
             updates.append((s, val))
 
         for var in theano.gof.graph.ancestors(update for _, update in updates):
-            if var.name is not None and var.name is not 'b':
+            if var.name is not None and var.name != 'b':
                 if var.name[0] != 's' or len(var.name) != 2:
                     var.name = None
 
