@@ -1,8 +1,8 @@
-import copy
+from __future__ import absolute_import, print_function, division
 
-import numpy
+import numpy as np
 
-from type import TypedListType
+from .type import TypedListType
 import theano
 from theano.gof import Apply, Constant, Op, Variable
 from theano.tensor.type_other import SliceType
@@ -48,20 +48,25 @@ class _typed_list_py_operators:
 class TypedListVariable(_typed_list_py_operators, Variable):
     """
     Subclass to add the typed list operators to the basic `Variable` class.
+
     """
 
 TypedListType.Variable = TypedListVariable
 
 
+class TypedListConstant(_typed_list_py_operators, Constant):
+    """
+    Subclass to add the typed list operators to the basic `Variable` class.
+
+    """
+
+TypedListType.Constant = TypedListConstant
+
+
 class GetItem(Op):
     # See doc in instance of this Op or function after this class definition.
     view_map = {0: [0]}
-
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
+    __props__ = ()
 
     def make_node(self, x, index):
         assert isinstance(x.type, TypedListType)
@@ -80,7 +85,9 @@ class GetItem(Op):
         else:
             raise TypeError('Expected scalar or slice as index.')
 
-    def perform(self, node, (x, index), (out, )):
+    def perform(self, node, inputs, outputs):
+        (x, index) = inputs
+        (out,) = outputs
         if not isinstance(index, slice):
             index = int(index)
         out[0] = x[index]
@@ -107,13 +114,20 @@ getitem = GetItem()
 """
 Get specified slice of a typed list.
 
-:param x: typed list.
-:param index: the index of the value to return from `x`.
+Parameters
+----------
+x
+    Typed list.
+index
+    The index of the value to return from `x`.
+
 """
 
 
 class Append(Op):
     # See doc in instance of this Op after the class definition.
+    __props__ = ("inplace",)
+
     def __init__(self, inplace=False):
         self.inplace = inplace
         if self.inplace:
@@ -126,18 +140,14 @@ class Append(Op):
             # self.view_map = {0: [0, 1]}
             self.view_map = {0: [0]}
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.inplace == other.inplace
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.inplace)
-
     def make_node(self, x, toAppend):
         assert isinstance(x.type, TypedListType)
-        assert x.ttype == toAppend.type
+        assert x.ttype == toAppend.type, (x.ttype, toAppend.type)
         return Apply(self, [x, toAppend], [x.type()])
 
-    def perform(self, node, (x, toAppend), (out, )):
+    def perform(self, node, inputs, outputs):
+        (x, toAppend) = inputs
+        (out,) = outputs
         if not self.inplace:
             out[0] = list(x)
         else:
@@ -179,13 +189,20 @@ append = Append()
 """
 Append an element at the end of another list.
 
-:param x: the base typed list.
-:param y: the element to append to `x`.
+Parameters
+----------
+x
+    The base typed list.
+y
+    The element to append to `x`.
+
 """
 
 
 class Extend(Op):
     # See doc in instance of this Op after the class definition.
+    __props__ = ("inplace",)
+
     def __init__(self, inplace=False):
         self.inplace = inplace
         if self.inplace:
@@ -198,18 +215,14 @@ class Extend(Op):
             # self.view_map = {0: [0, 1]}
             self.view_map = {0: [0]}
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.inplace == other.inplace
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.inplace)
-
     def make_node(self, x, toAppend):
         assert isinstance(x.type, TypedListType)
         assert x.type == toAppend.type
         return Apply(self, [x, toAppend], [x.type()])
 
-    def perform(self, node, (x, toAppend), (out, )):
+    def perform(self, node, inputs, outputs):
+        (x, toAppend) = inputs
+        (out,) = outputs
         if not self.inplace:
             out[0] = list(x)
         else:
@@ -257,13 +270,20 @@ extend = Extend()
 """
 Append all elements of a list at the end of another list.
 
-:param x: The typed list to extend.
-:param toAppend: The typed list that will be added at the end of `x`.
+Parameters
+----------
+x
+    The typed list to extend.
+toAppend
+    The typed list that will be added at the end of `x`.
+
 """
 
 
 class Insert(Op):
     # See doc in instance of this Op after the class definition.
+    __props__ = ("inplace",)
+
     def __init__(self, inplace=False):
         self.inplace = inplace
         if self.inplace:
@@ -276,12 +296,6 @@ class Insert(Op):
             # self.view_map = {0: [0, 2]}
             self.view_map = {0: [0]}
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.inplace == other.inplace
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.inplace)
-
     def make_node(self, x, index, toInsert):
         assert isinstance(x.type, TypedListType)
         assert x.ttype == toInsert.type
@@ -292,7 +306,9 @@ class Insert(Op):
             assert isinstance(index, T.TensorVariable) and index.ndim == 0
         return Apply(self, [x, index, toInsert], [x.type()])
 
-    def perform(self, node, (x, index, toInsert), (out, )):
+    def perform(self, node, inputs, outputs):
+        (x, index, toInsert) = inputs
+        (out,) = outputs
         if not self.inplace:
             out[0] = list(x)
         else:
@@ -334,14 +350,22 @@ insert = Insert()
 """
 Insert an element at an index in a typed list.
 
-:param x: the typed list to modify.
-:param index: the index where to put the new element in `x`.
-:param toInsert: The new element to insert.
+Parameters
+----------
+x
+    The typed list to modify.
+index
+    The index where to put the new element in `x`.
+toInsert
+    The new element to insert.
+
 """
 
 
 class Remove(Op):
     # See doc in instance of this Op after the class definition.
+    __props__ = ("inplace",)
+
     def __init__(self, inplace=False):
         self.inplace = inplace
         if self.inplace:
@@ -349,28 +373,23 @@ class Remove(Op):
         else:
             self.view_map = {0: [0]}
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.inplace == other.inplace
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.inplace)
-
     def make_node(self, x, toRemove):
         assert isinstance(x.type, TypedListType)
         assert x.ttype == toRemove.type
         return Apply(self, [x, toRemove], [x.type()])
 
-    def perform(self, node, (x, toRemove), (out, )):
-
+    def perform(self, node, inputs, outputs):
+        (x, toRemove) = inputs
+        (out,) = outputs
         if not self.inplace:
             out[0] = list(x)
         else:
             out[0] = x
 
         """
-        inelegant workaround for ValueError: The truth value of an
+        Inelegant workaround for ValueError: The truth value of an
         array with more than one element is ambiguous. Use a.any() or a.all()
-        being thrown when trying to remove a matrix from a matrices list
+        being thrown when trying to remove a matrix from a matrices list.
         """
         for y in range(out[0].__len__()):
                 if node.inputs[0].ttype.values_eq(out[0][y], toRemove):
@@ -383,19 +402,26 @@ class Remove(Op):
 remove = Remove()
 """Remove an element from a typed list.
 
-:param x: the typed list to be changed.
-:param toRemove: an element to be removed from the typed list.
+Parameters
+----------
+x
+    The typed list to be changed.
+toRemove
+    An element to be removed from the typed list.
     We only remove the first instance.
 
-:note: Python implementation of remove doesn't work when we want to
-    remove an ndarray from a list. This implementation works in that
-    case.
+Notes
+-----
+Python implementation of remove doesn't work when we want to remove an ndarray
+from a list. This implementation works in that case.
 
 """
 
 
 class Reverse(Op):
     # See doc in instance of this Op after the class definition.
+    __props__ = ("inplace",)
+
     def __init__(self, inplace=False):
         self.inplace = inplace
         if self.inplace:
@@ -403,18 +429,12 @@ class Reverse(Op):
         else:
             self.view_map = {0: [0]}
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.inplace == other.inplace
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.inplace)
-
     def make_node(self, x):
         assert isinstance(x.type, TypedListType)
         return Apply(self, [x], [x.type()])
 
-    def perform(self, node, inp, (out, )):
-
+    def perform(self, node, inp, outputs):
+        (out,) = outputs
         if not self.inplace:
             out[0] = list(inp[0])
         else:
@@ -453,32 +473,34 @@ reverse = Reverse()
 """
 Reverse the order of a typed list.
 
-:param x: the typed list to be reversed.
+Parameters
+----------
+x
+    The typed list to be reversed.
+
 """
 
 
 class Index(Op):
     # See doc in instance of this Op after the class definition.
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
+    __props__ = ()
 
     def make_node(self, x, elem):
         assert isinstance(x.type, TypedListType)
         assert x.ttype == elem.type
         return Apply(self, [x, elem], [T.scalar()])
 
-    def perform(self, node, (x, elem), (out, )):
+    def perform(self, node, inputs, outputs):
         """
-        inelegant workaround for ValueError: The truth value of an
+        Inelegant workaround for ValueError: The truth value of an
         array with more than one element is ambiguous. Use a.any() or a.all()
         being thrown when trying to remove a matrix from a matrices list
         """
+        (x, elem) = inputs
+        (out,) = outputs
         for y in range(len(x)):
             if node.inputs[0].ttype.values_eq(x[y], elem):
-                out[0] = numpy.asarray(y, dtype=theano.config.floatX)
+                out[0] = np.asarray(y, dtype=theano.config.floatX)
                 break
 
     def __str__(self):
@@ -489,28 +511,26 @@ index_ = Index()
 
 class Count(Op):
     # See doc in instance of this Op after the class definition.
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
+    __props__ = ()
 
     def make_node(self, x, elem):
         assert isinstance(x.type, TypedListType)
         assert x.ttype == elem.type
         return Apply(self, [x, elem], [T.scalar()])
 
-    def perform(self, node, (x, elem), (out, )):
+    def perform(self, node, inputs, outputs):
         """
-        inelegant workaround for ValueError: The truth value of an
+        Inelegant workaround for ValueError: The truth value of an
         array with more than one element is ambiguous. Use a.any() or a.all()
         being thrown when trying to remove a matrix from a matrices list
         """
+        (x, elem) = inputs
+        (out,) = outputs
         out[0] = 0
         for y in range(len(x)):
             if node.inputs[0].ttype.values_eq(x[y], elem):
                 out[0] += 1
-        out[0] = numpy.asarray(out[0], dtype=theano.config.floatX)
+        out[0] = np.asarray(out[0], dtype=theano.config.floatX)
 
     def __str__(self):
         return self.__class__.__name__
@@ -519,32 +539,33 @@ count = Count()
 """
 Count the number of times an element is in the typed list.
 
-:param x: The typed list to look into.
-:param elem: The element we want to count in list.
+Parameters
+----------
+x
+    The typed list to look into.
+elem
+    The element we want to count in list.
     The elements are compared with equals.
 
-:note: Python implementation of count doesn't work when we want to
-    count an ndarray from a list. This implementation works in that
-    case.
+Notes
+-----
+Python implementation of count doesn't work when we want to count an ndarray
+from a list. This implementation works in that case.
 
 """
 
 
 class Length(Op):
     # See doc in instance of this Op after the class definition.
-
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
+    __props__ = ()
 
     def make_node(self, x):
         assert isinstance(x.type, TypedListType)
         return Apply(self, [x], [T.scalar(dtype='int64')])
 
-    def perform(self, node, x, (out, )):
-        out[0] = numpy.asarray(len(x[0]), 'int64')
+    def perform(self, node, x, outputs):
+        (out,) = outputs
+        out[0] = np.asarray(len(x[0]), 'int64')
 
     def __str__(self):
         return self.__class__.__name__
@@ -567,24 +588,23 @@ length = Length()
 """
 Returns the size of a list.
 
-:param x: typed list.
+Parameters
+----------
+x
+    Typed list.
+
 """
 
 
 class MakeList(Op):
-
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
+    __props__ = ()
 
     def make_node(self, a):
         assert isinstance(a, (tuple, list))
         a2 = []
         for elem in a:
             if not isinstance(elem, theano.gof.Variable):
-                elem = as_tensor_variable(elem)
+                elem = theano.tensor.as_tensor_variable(elem)
             a2.append(elem)
         if not all(a2[0].type == elem.type for elem in a2):
             raise TypeError(
@@ -593,14 +613,21 @@ class MakeList(Op):
 
         return Apply(self, a2, [tl])
 
-    def perform(self, node, inputs, (out, )):
-        out[0] = list(inputs)
+    def perform(self, node, inputs, outputs):
+        (out,) = outputs
+        # We need to make sure that we don't get a view on our inputs
+        out[0] = [_lessbroken_deepcopy(inp) for inp in inputs]
 
 make_list = MakeList()
 """
 Build a Python list from those Theano variable.
 
-:param a: tuple/list of Theano variable
+Parameters
+----------
+a : tuple/list of Theano variable
+
+Notes
+-----
+All Theano variables must have the same type.
 
-:note: All Theano variable must have the same type.
 """
